@@ -43,6 +43,8 @@ typedef struct {
 static WebKitSettings *webkit_settings = NULL;
 static GFileMonitor *user_style_sheet_monitor = NULL;
 static WebKitUserStyleSheet *style_sheet = NULL;
+static GFileMonitor *user_javascript_monitor = NULL;
+static WebKitUserScript *javascript = NULL;
 static GList *ucm_list = NULL;
 
 static void
@@ -127,7 +129,11 @@ webkit_pref_callback_user_stylesheet (GSettings  *settings,
 
   value = g_settings_get_boolean (settings, key);
 
-  g_clear_object (&user_style_sheet_monitor);
+  if (user_style_sheet_monitor) {
+    g_signal_handlers_disconnect_by_func (user_style_sheet_monitor, user_style_sheet_file_changed, NULL);
+    g_clear_object (&user_style_sheet_monitor);
+  }
+
   g_clear_pointer (&style_sheet, webkit_user_style_sheet_unref);
 
   if (!value) {
@@ -150,6 +156,113 @@ webkit_pref_callback_user_stylesheet (GSettings  *settings,
   }
 }
 
+static void
+update_user_javascript_on_all_ucm (void)
+{
+  GList *list = NULL;
+
+  for (list = ucm_list; list != NULL; list = list->next) {
+    WebKitUserContentManager *ucm = list->data;
+
+    webkit_user_content_manager_remove_all_scripts (ucm);
+    if (javascript)
+      webkit_user_content_manager_add_script (ucm, javascript);
+  }
+}
+
+static void
+user_javascript_output_stream_splice_cb (GOutputStream *output_stream,
+                                         GAsyncResult  *result,
+                                         gpointer       user_data)
+{
+  gssize bytes;
+
+  g_clear_pointer (&javascript, webkit_user_script_unref);
+
+  bytes = g_output_stream_splice_finish (output_stream, result, NULL);
+  if (bytes > 0) {
+    javascript = webkit_user_script_new (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (output_stream)),
+                                         WEBKIT_USER_CONTENT_INJECT_ALL_FRAMES, WEBKIT_USER_SCRIPT_INJECT_AT_DOCUMENT_END,
+                                         NULL, NULL);
+  }
+
+  update_user_javascript_on_all_ucm ();
+
+  g_object_unref (output_stream);
+}
+
+static void
+user_javascript_read_cb (GFile        *file,
+                         GAsyncResult *result,
+                         gpointer      user_data)
+{
+  g_autoptr (GFileInputStream) input_stream = NULL;
+  g_autoptr (GOutputStream) output_stream = NULL;
+
+  input_stream = g_file_read_finish (file, result, NULL);
+  if (!input_stream)
+    return;
+
+  output_stream = g_memory_output_stream_new_resizable ();
+  g_output_stream_splice_async (g_steal_pointer (&output_stream),
+                                G_INPUT_STREAM (input_stream),
+                                G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+                                G_PRIORITY_DEFAULT,
+                                NULL,
+                                (GAsyncReadyCallback)user_javascript_output_stream_splice_cb,
+                                NULL);
+}
+
+static void
+user_javascript_file_changed (GFileMonitor      *monitor,
+                              GFile             *file,
+                              GFile             *other_file,
+                              GFileMonitorEvent  event_type,
+                              gpointer           user_data)
+{
+  if (event_type == G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT) {
+    g_file_read_async (file, G_PRIORITY_DEFAULT, NULL,
+                       (GAsyncReadyCallback)user_javascript_read_cb, NULL);
+  }
+}
+
+static void
+webkit_pref_callback_user_javascript (GSettings  *settings,
+                                      const char *key,
+                                      gpointer    data)
+{
+  g_autoptr (GFile) file = NULL;
+  g_autofree char *filename = NULL;
+  gboolean value;
+  g_autoptr (GError) error = NULL;
+
+  value = g_settings_get_boolean (settings, key);
+
+  if (user_javascript_monitor) {
+    g_signal_handlers_disconnect_by_func (user_javascript_monitor, user_javascript_file_changed, NULL);
+    g_clear_object (&user_style_sheet_monitor);
+  }
+
+  if (!value) {
+    update_user_javascript_on_all_ucm ();
+    return;
+  }
+
+  filename = g_build_filename (ephy_profile_dir (), USER_JAVASCRIPT_FILENAME, NULL);
+  file = g_file_new_for_path (filename);
+
+  g_file_read_async (file, G_PRIORITY_DEFAULT, NULL,
+                     (GAsyncReadyCallback)user_javascript_read_cb, NULL);
+
+  user_javascript_monitor = g_file_monitor_file (file, G_FILE_MONITOR_NONE, NULL, &error);
+  if (!user_javascript_monitor) {
+    g_warning ("Could not create a file monitor for %s: %s\n", g_file_get_uri (file), error->message);
+    g_error_free (error);
+  } else {
+    g_signal_connect (user_javascript_monitor, "changed", G_CALLBACK (user_javascript_file_changed), NULL);
+  }
+}
+
 static void
 webkit_pref_callback_user_agent (GSettings  *settings,
                                  const char *key,
@@ -166,19 +279,7 @@ webkit_pref_callback_font_size (GSettings  *settings,
 {
   char *webkit_pref = data;
   char *value = NULL;
-  int size = 9; /* FIXME: What to use here? */
-
-  char *schema = NULL;
-  g_object_get (settings, "schema-id", &schema, NULL);
-
-  /* If we are changing a GNOME font value and we are not using GNOME fonts in
-   * Epiphany, return. */
-  if (g_strcmp0 (schema, EPHY_PREFS_WEB_SCHEMA) != 0 &&
-      g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_USE_GNOME_FONTS) != TRUE) {
-    g_free (schema);
-    return;
-  }
-  g_free (schema);
+  int size = 12;
 
   value = g_settings_get_string (settings, key);
 
@@ -204,18 +305,6 @@ webkit_pref_callback_font_family (GSettings  *settings,
   char *webkit_pref = data;
   char *value = NULL;
 
-  char *schema = NULL;
-  g_object_get (settings, "schema-id", &schema, NULL);
-
-  /* If we are changing a GNOME font value and we are not using GNOME fonts in
-   * Epiphany, return. */
-  if (g_strcmp0 (schema, EPHY_PREFS_WEB_SCHEMA) != 0 &&
-      g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_USE_GNOME_FONTS) != TRUE) {
-    g_free (schema);
-    return;
-  }
-  g_free (schema);
-
   value = g_settings_get_string (settings, key);
 
   if (value) {
@@ -303,46 +392,6 @@ webkit_pref_callback_accept_languages (GSettings  *settings,
   g_array_free (array, TRUE);
 }
 
-
-void
-ephy_embed_prefs_set_cookie_accept_policy (WebKitCookieManager *cookie_manager,
-                                           const char          *settings_policy)
-{
-  WebKitCookieAcceptPolicy policy;
-
-  if (!strcmp (settings_policy, "never"))
-    policy = WEBKIT_COOKIE_POLICY_ACCEPT_NEVER;
-  else if (!strcmp (settings_policy, "always"))
-    policy = WEBKIT_COOKIE_POLICY_ACCEPT_ALWAYS;
-  else if (!strcmp (settings_policy, "no-third-party"))
-    policy = WEBKIT_COOKIE_POLICY_ACCEPT_NO_THIRD_PARTY;
-  else {
-    g_warn_if_reached ();
-    return;
-  }
-
-  webkit_cookie_manager_set_accept_policy (cookie_manager, policy);
-}
-
-static void
-webkit_pref_callback_cookie_accept_policy (GSettings  *settings,
-                                           const char *key,
-                                           gpointer    data)
-{
-  WebKitCookieManager *cookie_manager;
-  char *value;
-  EphyEmbedShell *shell;
-
-  value = g_settings_get_string (settings, key);
-  if (!value)
-    return;
-
-  shell = ephy_embed_shell_get_default ();
-  cookie_manager = webkit_web_context_get_cookie_manager (ephy_embed_shell_get_web_context (shell));
-  ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, value);
-  g_free (value);
-}
-
 static void
 webkit_pref_callback_gnome_fonts (GSettings  *ephy_settings,
                                   const char *key,
@@ -351,6 +400,7 @@ webkit_pref_callback_gnome_fonts (GSettings  *ephy_settings,
   if (g_settings_get_boolean (ephy_settings, key)) {
     g_object_set (webkit_settings,
                   "default-font-family", "serif",
+                  "serif-font-family", "serif",
                   "sans-serif-font-family", "sans-serif",
                   "monospace-font-family", "monospace",
                   "default-font-size", webkit_settings_font_size_to_pixels (12),
@@ -399,6 +449,26 @@ webkit_pref_callback_enable_spell_checking (GSettings  *settings,
   }
 }
 
+static void
+webkit_pref_callback_enable_website_data_storage (GSettings  *settings,
+                                                  const char *key,
+                                                  gpointer    data)
+{
+  EphyEmbedShell *shell;
+  WebKitWebContext *context;
+  WebKitCookieManager *manager;
+  gboolean value;
+
+  value = g_settings_get_boolean (settings, key);
+  webkit_settings_set_enable_html5_database (webkit_settings, value);
+  webkit_settings_set_enable_html5_local_storage (webkit_settings, value);
+
+  shell = ephy_embed_shell_get_default ();
+  context = ephy_embed_shell_get_web_context (shell);
+  manager = webkit_web_context_get_cookie_manager (context);
+  webkit_cookie_manager_set_accept_policy (manager, value ? WEBKIT_COOKIE_POLICY_ACCEPT_ALWAYS : WEBKIT_COOKIE_POLICY_ACCEPT_NEVER);
+}
+
 static void
 webkit_pref_callback_hardware_acceleration_policy (GSettings  *settings,
                                                    const char *key,
@@ -466,6 +536,10 @@ static const PrefData webkit_pref_entries[] = {
     EPHY_PREFS_WEB_ENABLE_USER_CSS,
     "user-stylesheet-uri",
     webkit_pref_callback_user_stylesheet },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_ENABLE_USER_JS,
+    "user-javascript-uri",
+    webkit_pref_callback_user_javascript },
   { EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_LANGUAGE,
     "accept-language",
@@ -475,14 +549,9 @@ static const PrefData webkit_pref_entries[] = {
     "user-agent",
     webkit_pref_callback_user_agent },
   { EPHY_PREFS_WEB_SCHEMA,
-    EPHY_PREFS_WEB_MOBILE_USER_AGENT,
-    "mobile-user-agent",
-    /* No need for a specific callback, we just want to update the user agent. */
-    webkit_pref_callback_user_agent },
-  { EPHY_PREFS_WEB_SCHEMA,
-    EPHY_PREFS_WEB_COOKIES_POLICY,
-    "accept-policy",
-    webkit_pref_callback_cookie_accept_policy },
+    EPHY_PREFS_WEB_ENABLE_WEBSITE_DATA_STORAGE,
+    NULL,
+    webkit_pref_callback_enable_website_data_storage },
   { EPHY_PREFS_WEB_SCHEMA,
     EPHY_PREFS_WEB_HARDWARE_ACCELERATION_POLICY,
     "hardware-acceleration-policy",
@@ -498,12 +567,6 @@ ephy_embed_prefs_init (gpointer user_data)
                                                        "enable-developer-extras", TRUE,
                                                        "enable-dns-prefetching", TRUE,
                                                        "enable-plugins", FALSE,
-                                                       "enable-webaudio", TRUE, /* FIXME: https://bugs.webkit.org/show_bug.cgi?id=205334 */
-                                                       "enable-webgl", TRUE, /* FIXME: https://bugs.webkit.org/show_bug.cgi?id=205335 */
-                                                       "javascript-can-open-windows-automatically", TRUE,
-#if WEBKIT_CHECK_VERSION (2, 27, 90) && !WEBKIT_CHECK_VERSION (2, 30, 0)
-                                                       "allow-top-navigation-to-data-urls", FALSE,
-#endif
                                                        NULL);
 
   for (i = 0; i < G_N_ELEMENTS (webkit_pref_entries); i++) {
@@ -561,6 +624,13 @@ ephy_embed_prefs_apply_user_style (WebKitUserContentManager *ucm)
     webkit_user_content_manager_add_style_sheet (ucm, style_sheet);
 }
 
+void
+ephy_embed_prefs_apply_user_javascript (WebKitUserContentManager *ucm)
+{
+  if (javascript)
+    webkit_user_content_manager_add_script (ucm, javascript);
+}
+
 void
 ephy_embed_prefs_register_ucm (WebKitUserContentManager *ucm)
 {
