@@ -35,7 +35,9 @@
 #include "ephy-flatpak-utils.h"
 #include "ephy-history-service.h"
 #include "ephy-password-manager.h"
+#include "ephy-pdf-handler.h"
 #include "ephy-profile-utils.h"
+#include "ephy-reader-handler.h"
 #include "ephy-settings.h"
 #include "ephy-snapshot-service.h"
 #include "ephy-tabs-catalog.h"
@@ -63,6 +65,8 @@ typedef struct {
   EphyPasswordManager *password_manager;
   EphyAboutHandler *about_handler;
   EphyViewSourceHandler *source_handler;
+  EphyReaderHandler *reader_handler;
+  EphyPDFHandler *pdf_handler;
   char *guid;
   EphyFiltersManager *filters_manager;
   EphySearchEngineManager *search_engine_manager;
@@ -188,7 +192,9 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->global_history_service);
   g_clear_object (&priv->global_gsb_service);
   g_clear_object (&priv->about_handler);
+  g_clear_object (&priv->reader_handler);
   g_clear_object (&priv->source_handler);
+  g_clear_object (&priv->pdf_handler);
   g_clear_object (&priv->downloads_manager);
   g_clear_object (&priv->password_manager);
   g_clear_object (&priv->permissions_manager);
@@ -629,7 +635,7 @@ ephy_embed_shell_get_global_gsb_service (EphyEmbedShell *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
-  g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
+  g_assert (EPHY_IS_EMBED_SHELL (shell));
 
   if (!priv->global_gsb_service) {
     g_autofree char *api_key = NULL;
@@ -686,6 +692,24 @@ source_request_cb (WebKitURISchemeRequest *request,
   ephy_view_source_handler_handle_request (priv->source_handler, request);
 }
 
+static void
+reader_request_cb (WebKitURISchemeRequest *request,
+                   EphyEmbedShell         *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  ephy_reader_handler_handle_request (priv->reader_handler, request);
+}
+
+static void
+pdf_request_cb (WebKitURISchemeRequest *request,
+                EphyEmbedShell         *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  ephy_pdf_handler_handle_request (priv->pdf_handler, request);
+}
+
 static void
 ephy_resource_request_cb (WebKitURISchemeRequest *request)
 {
@@ -757,8 +781,13 @@ ephy_embed_shell_create_web_context (EphyEmbedShell *shell)
     manager = webkit_website_data_manager_new ("base-data-directory", ephy_profile_dir (),
                                                "base-cache-directory", ephy_cache_dir (),
                                                NULL);
+    webkit_website_data_manager_set_persistent_credential_storage_enabled (manager, FALSE);
   }
 
+  webkit_website_data_manager_set_itp_enabled (manager,
+                                               g_settings_get_boolean (EPHY_SETTINGS_WEB,
+                                                                       EPHY_PREFS_WEB_ENABLE_ITP));
+
   priv->web_context = g_object_new (WEBKIT_TYPE_WEB_CONTEXT,
                                     "website-data-manager", manager,
                                     "process-swap-on-cross-site-navigation-enabled", TRUE,
@@ -798,12 +827,7 @@ download_started_cb (WebKitWebContext *web_context,
 
   web_view = webkit_download_get_web_view (download);
   if (EPHY_IS_WEB_VIEW (web_view)) {
-    EphyEmbed *embed;
-
-    embed = EPHY_GET_EMBED_FROM_EPHY_WEB_VIEW (web_view);
-    ephy_embed_download_started (embed, ephy_download);
-
-    if (!ephy_web_view_in_pdf_viewer (EPHY_WEB_VIEW (web_view)))
+    if (ephy_web_view_get_document_type (EPHY_WEB_VIEW (web_view)) != EPHY_WEB_VIEW_DOCUMENT_PDF)
       ephy_downloads_manager_add_download (priv->downloads_manager, ephy_download);
   } else {
     ephy_downloads_manager_add_download (priv->downloads_manager, ephy_download);
@@ -825,6 +849,20 @@ remember_passwords_setting_changed_cb (GSettings      *settings,
                                                                               g_variant_new ("b", should_remember_passwords)));
 }
 
+static void
+enable_itp_setting_changed_cb (GSettings      *settings,
+                               char           *key,
+                               EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+  WebKitWebsiteDataManager *manager;
+
+  manager = webkit_web_context_get_website_data_manager (priv->web_context);
+  webkit_website_data_manager_set_itp_enabled (manager,
+                                               g_settings_get_boolean (EPHY_SETTINGS_WEB,
+                                                                       EPHY_PREFS_WEB_ENABLE_ITP));
+}
+
 static void
 ephy_embed_shell_startup (GApplication *application)
 {
@@ -833,12 +871,9 @@ ephy_embed_shell_startup (GApplication *application)
   g_autofree char *favicon_db_path = NULL;
   WebKitCookieManager *cookie_manager;
   g_autofree char *filename = NULL;
-  g_autofree char *cookie_policy = NULL;
 
   G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->startup (application);
 
-  ephy_embed_shell_create_web_context (shell);
-
   webkit_web_context_set_process_model (priv->web_context, WEBKIT_PROCESS_MODEL_MULTIPLE_SECONDARY_PROCESSES);
 
   webkit_web_context_set_sandbox_enabled (priv->web_context, TRUE);
@@ -854,7 +889,6 @@ ephy_embed_shell_startup (GApplication *application)
                            G_CALLBACK (initialize_web_process_extensions),
                            shell, 0);
 
-  priv->permissions_manager = ephy_permissions_manager_new ();
   g_signal_connect_object (priv->web_context, "initialize-notification-permissions",
                            G_CALLBACK (initialize_notification_permissions),
                            shell, 0);
@@ -889,6 +923,17 @@ ephy_embed_shell_startup (GApplication *application)
   webkit_web_context_register_uri_scheme (priv->web_context, EPHY_VIEW_SOURCE_SCHEME,
                                           (WebKitURISchemeRequestCallback)source_request_cb,
                                           shell, NULL);
+  /* pdf handler */
+  priv->pdf_handler = ephy_pdf_handler_new ();
+  webkit_web_context_register_uri_scheme (priv->web_context, EPHY_PDF_SCHEME,
+                                          (WebKitURISchemeRequestCallback)pdf_request_cb,
+                                          shell, NULL);
+
+  /* reader mode handler */
+  priv->reader_handler = ephy_reader_handler_new ();
+  webkit_web_context_register_uri_scheme (priv->web_context, EPHY_READER_SCHEME,
+                                          (WebKitURISchemeRequestCallback)reader_request_cb,
+                                          shell, NULL);
 
   /* ephy-resource handler */
   webkit_web_context_register_uri_scheme (priv->web_context, "ephy-resource",
@@ -905,17 +950,14 @@ ephy_embed_shell_startup (GApplication *application)
                                                   WEBKIT_COOKIE_PERSISTENT_STORAGE_SQLITE);
   }
 
-  cookie_policy = g_settings_get_string (EPHY_SETTINGS_WEB,
-                                         EPHY_PREFS_WEB_COOKIES_POLICY);
-  ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, cookie_policy);
-
-  priv->filters_manager = ephy_filters_manager_new (NULL);
-
   g_signal_connect_object (priv->web_context, "download-started",
                            G_CALLBACK (download_started_cb), shell, 0);
 
   g_signal_connect_object (EPHY_SETTINGS_WEB, "changed::remember-passwords",
                            G_CALLBACK (remember_passwords_setting_changed_cb), shell, 0);
+
+  g_signal_connect_object (EPHY_SETTINGS_WEB, "changed::enable-itp",
+                           G_CALLBACK (enable_itp_setting_changed_cb), shell, 0);
 }
 
 static void
@@ -966,21 +1008,17 @@ ephy_embed_shell_constructed (GObject *object)
 {
   EphyEmbedShell *shell;
   EphyEmbedShellPrivate *priv;
-  EphyEmbedShellMode mode;
 
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->constructed (object);
 
   shell = EPHY_EMBED_SHELL (object);
   priv = ephy_embed_shell_get_instance_private (shell);
   priv->guid = g_dbus_generate_guid ();
-  mode = ephy_embed_shell_get_mode (shell);
 
-  /* These do not run the EmbedShell application instance, so make sure that
-   *  there is a web context and a user content manager for them. */
-  if (mode == EPHY_EMBED_SHELL_MODE_TEST ||
-      mode == EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER) {
-    ephy_embed_shell_create_web_context (shell);
-  }
+  ephy_embed_shell_create_web_context (shell);
+
+  priv->permissions_manager = ephy_permissions_manager_new ();
+  priv->filters_manager = ephy_filters_manager_new (NULL);
 }
 
 static void
@@ -1382,6 +1420,7 @@ ephy_embed_shell_register_ucm_handler (EphyEmbedShell           *shell,
 
   /* User Scripts */
   ephy_embed_prefs_apply_user_style (ucm);
+  ephy_embed_prefs_apply_user_javascript (ucm);
 }
 
 void
@@ -1408,3 +1447,12 @@ ephy_embed_shell_unregister_ucm_handler (EphyEmbedShell           *shell,
                                                                           "passwordManagerSave",
                                                                           priv->guid);
 }
+
+void
+ephy_embed_shell_pdf_handler_stop (EphyEmbedShell *shell,
+                                   WebKitWebView  *web_view)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+
+  ephy_pdf_handler_stop (priv->pdf_handler, web_view);
+}
