@@ -23,7 +23,7 @@
 #include "ephy-find-toolbar.h"
 
 #include "ephy-debug.h"
-#include "ephy-web-view.h"
+#include "contrib/gd-tagged-entry.h"
 
 #include <math.h>
 
@@ -36,19 +36,20 @@
 struct _EphyFindToolbar {
   HdySearchBar parent_instance;
 
+  GCancellable *cancellable;
   WebKitWebView *web_view;
   WebKitFindController *controller;
-  GtkWidget *entry;
+  GdTaggedEntry *entry;
+  GdTaggedEntryTag *entry_tag;
   GtkWidget *next;
   GtkWidget *prev;
+  guint num_matches;
+  guint current_match;
   guint find_again_source_id;
   guint find_source_id;
   char *find_string;
-  guint preedit_changed : 1;
-  guint prevent_activate : 1;
-  guint activated : 1;
-  guint links_only : 1;
-  guint typing_ahead : 1;
+  gboolean links_only;
+  gboolean typing_ahead;
 };
 
 G_DEFINE_TYPE (EphyFindToolbar, ephy_find_toolbar, HDY_TYPE_SEARCH_BAR)
@@ -81,7 +82,18 @@ typedef enum {
 
 /* private functions */
 
-static void ephy_find_toolbar_set_web_view (EphyFindToolbar *toolbar, WebKitWebView *web_view);
+static void ephy_find_toolbar_set_web_view (EphyFindToolbar *toolbar,
+                                            WebKitWebView   *web_view);
+
+static void
+update_search_tag (EphyFindToolbar *toolbar)
+{
+  g_autofree gchar *label = NULL;
+
+  label = g_strdup_printf ("%u/%u", toolbar->current_match, toolbar->num_matches);
+  gd_tagged_entry_tag_set_label (toolbar->entry_tag, label);
+  gd_tagged_entry_add_tag (toolbar->entry, toolbar->entry_tag);
+}
 
 static void
 set_status (EphyFindToolbar *toolbar,
@@ -90,6 +102,8 @@ set_status (EphyFindToolbar *toolbar,
   const char *icon_name = "edit-find-symbolic";
   const char *tooltip = NULL;
 
+  update_search_tag (toolbar);
+
   switch (result) {
     case EPHY_FIND_RESULT_FOUND:
       break;
@@ -126,6 +140,8 @@ clear_status (EphyFindToolbar *toolbar)
                 "primary-icon-name", "edit-find-symbolic",
                 NULL);
 
+  gd_tagged_entry_remove_tag (toolbar->entry, toolbar->entry_tag);
+
   gtk_widget_set_sensitive (toolbar->prev, FALSE);
   gtk_widget_set_sensitive (toolbar->next, FALSE);
 
@@ -135,8 +151,8 @@ clear_status (EphyFindToolbar *toolbar)
 }
 
 static void
-real_find (EphyFindToolbar  *toolbar,
-           EphyFindDirection direction)
+real_find (EphyFindToolbar   *toolbar,
+           EphyFindDirection  direction)
 {
   WebKitFindOptions options = WEBKIT_FIND_OPTIONS_CASE_INSENSITIVE;
 
@@ -146,6 +162,7 @@ real_find (EphyFindToolbar  *toolbar,
   if (direction == EPHY_FIND_DIRECTION_PREV)
     options |= WEBKIT_FIND_OPTIONS_BACKWARDS;
 
+  webkit_find_controller_count_matches (toolbar->controller, toolbar->find_string, options, G_MAXUINT);
   webkit_find_controller_search (toolbar->controller, toolbar->find_string, options, G_MAXUINT);
 }
 
@@ -167,9 +184,11 @@ found_text_cb (WebKitFindController *controller,
   WebKitFindOptions options;
   EphyFindResult result;
 
+  update_search_tag (toolbar);
+
   options = webkit_find_controller_get_options (controller);
   /* FIXME: it's not possible to remove the wrap flag, so the status is now always wrapped. */
-  result = options & WEBKIT_FIND_OPTIONS_WRAP_AROUND ? EPHY_FIND_RESULT_FOUNDWRAPPED : EPHY_FIND_RESULT_FOUND;
+  result = (options & WEBKIT_FIND_OPTIONS_WRAP_AROUND) ? EPHY_FIND_RESULT_FOUNDWRAPPED : EPHY_FIND_RESULT_FOUND;
   set_status (toolbar, result);
 }
 
@@ -195,10 +214,7 @@ update_find_string (EphyFindToolbar *toolbar)
   g_free (toolbar->find_string);
   toolbar->find_string = g_strdup (gtk_entry_get_text (GTK_ENTRY (toolbar->entry)));
 
-  if (toolbar->find_source_id != 0) {
-    g_source_remove (toolbar->find_source_id);
-    toolbar->find_source_id = 0;
-  }
+  g_clear_handle_id (&toolbar->find_source_id, g_source_remove);
 
   if (strlen (toolbar->find_string) == 0) {
     clear_status (toolbar);
@@ -240,6 +256,14 @@ entry_key_press_event_cb (GtkEntry        *entry,
               event->keyval == GDK_KEY_KP_Enter ||
               event->keyval == GDK_KEY_ISO_Enter)) {
     handled = ephy_find_toolbar_activate_link (toolbar, event->state);
+  } else if ((event->state & mask) == GDK_CONTROL_MASK &&
+             (gdk_keyval_to_lower (event->keyval) == GDK_KEY_g)) {
+    handled = TRUE;
+    ephy_find_toolbar_find_next (toolbar);
+  } else if ((event->state & mask) == (GDK_CONTROL_MASK | GDK_SHIFT_MASK) &&
+             (gdk_keyval_to_lower (event->keyval) == GDK_KEY_g)) {
+    handled = TRUE;
+    ephy_find_toolbar_find_previous (toolbar);
   } else if ((event->state & mask) == GDK_SHIFT_MASK &&
              (event->keyval == GDK_KEY_Return ||
               event->keyval == GDK_KEY_KP_Enter ||
@@ -267,7 +291,7 @@ ephy_find_toolbar_grab_focus (GtkWidget *widget)
 {
   EphyFindToolbar *toolbar = EPHY_FIND_TOOLBAR (widget);
 
-  gtk_widget_grab_focus (toolbar->entry);
+  gtk_widget_grab_focus (GTK_WIDGET (toolbar->entry));
 }
 
 static gboolean
@@ -346,29 +370,33 @@ ephy_find_toolbar_load_changed_cb (WebKitWebView   *web_view,
 static void
 ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 {
-  GtkWidget *column;
+  GtkWidget *clamp;
   GtkWidget *box;
   GtkSizeGroup *size_group;
 
   size_group = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
 
-  column = GTK_WIDGET (hdy_column_new ());
-  hdy_column_set_maximum_width (HDY_COLUMN (column), 400);
-  hdy_column_set_linear_growth_width (HDY_COLUMN (column), 300);
-  gtk_container_add (GTK_CONTAINER (toolbar), column);
+  clamp = GTK_WIDGET (hdy_clamp_new ());
+  hdy_clamp_set_maximum_size (HDY_CLAMP (clamp), 400);
+  hdy_clamp_set_tightening_threshold (HDY_CLAMP (clamp), 300);
+  gtk_container_add (GTK_CONTAINER (toolbar), clamp);
 
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_style_context_add_class (gtk_widget_get_style_context (box),
                                GTK_STYLE_CLASS_RAISED);
   gtk_style_context_add_class (gtk_widget_get_style_context (box),
                                GTK_STYLE_CLASS_LINKED);
-  gtk_container_add (GTK_CONTAINER (column), box);
+  gtk_container_add (GTK_CONTAINER (clamp), box);
+
+  toolbar->entry = gd_tagged_entry_new ();
+  toolbar->entry_tag = gd_tagged_entry_tag_new ("");
+  gd_tagged_entry_tag_set_style (toolbar->entry_tag, "search-entry-occurrences-tag");
+  gd_tagged_entry_tag_set_has_close_button (toolbar->entry_tag, FALSE);
 
-  toolbar->entry = gtk_entry_new ();
   gtk_widget_set_hexpand (GTK_WIDGET (toolbar->entry), TRUE);
   gtk_entry_set_max_length (GTK_ENTRY (toolbar->entry), 512);
   gtk_entry_set_placeholder_text (GTK_ENTRY (toolbar->entry), _("Type to search…"));
-  gtk_container_add (GTK_CONTAINER (box), toolbar->entry);
+  gtk_container_add (GTK_CONTAINER (box), GTK_WIDGET (toolbar->entry));
 
   /* Prev */
   toolbar->prev = gtk_button_new_from_icon_name ("go-up-symbolic", GTK_ICON_SIZE_MENU);
@@ -385,7 +413,7 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
   gtk_container_add (GTK_CONTAINER (box), toolbar->next);
   gtk_widget_set_sensitive (toolbar->next, FALSE);
 
-  gtk_size_group_add_widget (size_group, toolbar->entry);
+  gtk_size_group_add_widget (size_group, GTK_WIDGET (toolbar->entry));
   gtk_size_group_add_widget (size_group, toolbar->next);
   gtk_size_group_add_widget (size_group, toolbar->prev);
   g_object_unref (size_group);
@@ -408,6 +436,8 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 
   search_entry_changed_cb (GTK_ENTRY (toolbar->entry), toolbar);
 
+  toolbar->cancellable = g_cancellable_new ();
+
   gtk_widget_show_all (GTK_WIDGET (toolbar));
 }
 
@@ -426,6 +456,11 @@ ephy_find_toolbar_dispose (GObject *object)
     toolbar->find_source_id = 0;
   }
 
+  if (toolbar->cancellable) {
+    g_cancellable_cancel (toolbar->cancellable);
+    g_clear_object (&toolbar->cancellable);
+  }
+
   G_OBJECT_CLASS (ephy_find_toolbar_parent_class)->dispose (object);
 }
 
@@ -516,6 +551,19 @@ ephy_find_toolbar_get_text (EphyFindToolbar *toolbar)
   return gtk_entry_get_text (GTK_ENTRY (toolbar->entry));
 }
 
+static void
+counted_matches_cb (WebKitFindController *find_controller,
+                    guint                 match_count,
+                    gpointer              user_data)
+{
+  EphyFindToolbar *toolbar = EPHY_FIND_TOOLBAR (user_data);
+
+  toolbar->num_matches = match_count;
+  toolbar->current_match = toolbar->num_matches ? 1 : 0;
+
+  update_search_tag (toolbar);
+}
+
 static void
 ephy_find_toolbar_set_web_view (EphyFindToolbar *toolbar,
                                 WebKitWebView   *web_view)
@@ -537,6 +585,9 @@ ephy_find_toolbar_set_web_view (EphyFindToolbar *toolbar,
     g_signal_connect_object (toolbar->controller, "failed-to-find-text",
                              G_CALLBACK (failed_to_find_text_cb),
                              toolbar, 0);
+    g_signal_connect_object (toolbar->controller, "counted_matches",
+                             G_CALLBACK (counted_matches_cb),
+                             toolbar, 0);
     g_signal_connect (web_view, "load-changed",
                       G_CALLBACK (ephy_find_toolbar_load_changed_cb),
                       toolbar);
@@ -548,15 +599,59 @@ ephy_find_toolbar_set_web_view (EphyFindToolbar *toolbar,
 void
 ephy_find_toolbar_find_next (EphyFindToolbar *toolbar)
 {
+  if (toolbar->num_matches) {
+    toolbar->current_match++;
+    if (toolbar->current_match > toolbar->num_matches)
+      toolbar->current_match = 1;
+  }
+
   webkit_find_controller_search_next (toolbar->controller);
 }
 
 void
 ephy_find_toolbar_find_previous (EphyFindToolbar *toolbar)
 {
+  if (toolbar->num_matches) {
+    g_assert (toolbar->current_match > 0);
+    toolbar->current_match--;
+    if (toolbar->current_match < 1)
+      toolbar->current_match = toolbar->num_matches;
+  }
+
   webkit_find_controller_search_previous (toolbar->controller);
 }
 
+static void
+ephy_find_toolbar_selection_async (GObject      *source_object,
+                                   GAsyncResult *res,
+                                   gpointer      user_data)
+{
+  WebKitWebView *web_view = WEBKIT_WEB_VIEW (source_object);
+  EphyFindToolbar *toolbar = EPHY_FIND_TOOLBAR (user_data);
+  g_autoptr (GError) error = NULL;
+  g_autoptr (WebKitJavascriptResult) js_result = NULL;
+  JSCValue *value = NULL;
+
+  js_result = webkit_web_view_run_javascript_finish (web_view, res, &error);
+  if (!js_result) {
+    g_warning ("Error running javascript: %s", error->message);
+    return;
+  }
+
+  value = webkit_javascript_result_get_js_value (js_result);
+  if (jsc_value_is_string (value)) {
+    JSCException *exception;
+    g_autofree gchar *str_value = NULL;
+
+    str_value = jsc_value_to_string (value);
+    exception = jsc_context_get_exception (jsc_value_get_context (value));
+    if (exception)
+      g_warning ("Error running javascript: %s", jsc_exception_get_message (exception));
+    else if (strlen (str_value))
+      gtk_entry_set_text (GTK_ENTRY (toolbar->entry), str_value);
+  }
+}
+
 void
 ephy_find_toolbar_open (EphyFindToolbar *toolbar,
                         gboolean         links_only,
@@ -567,11 +662,13 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
   toolbar->typing_ahead = typing_ahead;
   toolbar->links_only = links_only;
 
+  webkit_web_view_run_javascript (toolbar->web_view, "window.getSelection().toString();", toolbar->cancellable, ephy_find_toolbar_selection_async, toolbar);
+
   gtk_editable_select_region (GTK_EDITABLE (toolbar->entry), 0, -1);
 
   hdy_search_bar_set_search_mode (HDY_SEARCH_BAR (toolbar), TRUE);
   hdy_search_bar_set_show_close_button (HDY_SEARCH_BAR (toolbar), TRUE);
-  gtk_widget_grab_focus (toolbar->entry);
+  gtk_widget_grab_focus (GTK_WIDGET (toolbar->entry));
 }
 
 void
