@@ -43,7 +43,8 @@ G_DECLARE_FINAL_TYPE (EphyWebView, ephy_web_view, EPHY, WEB_VIEW, WebKitWebView)
                                         "^file:.*$|" \
                                         "^inspector://.*$|" \
                                         "^webkit://.*$|" \
-                                        "^ephy-resource://.*$" \
+                                        "^ephy-resource://.*$|" \
+                                        "^ephy-reader:.*$" \
                                         ")"
 
 #define EPHY_WEB_VIEW_DOMAIN_REGEX "^localhost(\\.[^[:space:]]+)?(:\\d+)?(:[0-9]+)?(/.*)?$|" \
@@ -60,6 +61,7 @@ typedef enum
   EPHY_WEB_VIEW_DOCUMENT_HTML,
   EPHY_WEB_VIEW_DOCUMENT_XML,
   EPHY_WEB_VIEW_DOCUMENT_IMAGE,
+  EPHY_WEB_VIEW_DOCUMENT_PDF,
   EPHY_WEB_VIEW_DOCUMENT_OTHER
 } EphyWebViewDocumentType;
 
@@ -143,6 +145,13 @@ void                       ephy_web_view_get_web_app_title        (EphyWebView
 char                      *ephy_web_view_get_web_app_title_finish (EphyWebView               *view,
                                                                    GAsyncResult              *result,
                                                                    GError                   **error);
+void                       ephy_web_view_get_web_app_mobile_capable        (EphyWebView         *view,
+                                                                            GCancellable        *cancellable,
+                                                                            GAsyncReadyCallback  callback,
+                                                                            gpointer             user_data);
+gboolean                   ephy_web_view_get_web_app_mobile_capable_finish (EphyWebView   *view,
+                                                                            GAsyncResult  *result,
+                                                                            GError       **error);
 
 void                       ephy_web_view_set_visit_type           (EphyWebView *view, 
                                                                    EphyHistoryPageVisitType visit_type);
@@ -173,6 +182,4 @@ void                       ephy_web_view_show_auth_form_save_request (EphyWebVie
                                                                       gpointer                        response_data,
                                                                       GDestroyNotify                  response_destroy);
 
-gboolean                   ephy_web_view_in_pdf_viewer             (EphyWebView              *view);
-
 G_END_DECLS
