@@ -54,7 +54,8 @@ build_modules (const char *name,
   *is_all = FALSE;
 
   env = g_getenv (name);
-  if (env == NULL) return NULL;
+  if (env == NULL)
+    return NULL;
 
   if (strcmp (env, "all") == 0) {
     *is_all = TRUE;
@@ -68,14 +69,15 @@ static char **ephy_log_modules;
 static gboolean ephy_log_all_modules;
 
 static void
-log_module (const gchar   *log_domain,
-            GLogLevelFlags log_level,
-            const char    *message,
-            gpointer       user_data)
+log_module (const gchar    *log_domain,
+            GLogLevelFlags  log_level,
+            const char     *message,
+            gpointer        user_data)
 {
   gboolean should_log = ephy_log_all_modules;
 
-  if (!ephy_log_all_modules && !ephy_log_modules) return;
+  if (!ephy_log_all_modules && !ephy_log_modules)
+    return;
 
   if (ephy_log_modules != NULL) {
     guint i;
@@ -88,18 +90,17 @@ log_module (const gchar   *log_domain,
     }
   }
 
-  if (should_log) {
+  if (should_log)
     g_print ("%s\n", message);
-  }
 }
 
 #define MAX_DEPTH 200
 
 static void
-trap_handler (const char    *log_domain,
-              GLogLevelFlags log_level,
-              const char    *message,
-              gpointer       user_data)
+trap_handler (const char     *log_domain,
+              GLogLevelFlags  log_level,
+              const char     *message,
+              gpointer        user_data)
 {
   g_log_default_handler (log_domain, log_level, message, user_data);
 
@@ -141,7 +142,8 @@ trap_handler (const char    *log_domain,
 }
 
 static EphyProfiler *
-ephy_profiler_new (const char *name, const char *module)
+ephy_profiler_new (const char *name,
+                   const char *module)
 {
   EphyProfiler *profiler;
 
@@ -165,7 +167,8 @@ ephy_should_profile (const char *module)
   slash = strrchr (module, '/');
 
   /* Happens on builddir != srcdir builds */
-  if (slash != NULL) module = slash + 1;
+  if (slash != NULL)
+    module = slash + 1;
 
   for (i = 0; ephy_profile_modules[i] != NULL; i++) {
     if (strcmp (ephy_profile_modules[i], module) == 0) {
@@ -182,7 +185,7 @@ ephy_profiler_dump (EphyProfiler *profiler)
 {
   double seconds;
 
-  g_return_if_fail (profiler != NULL);
+  g_assert (profiler != NULL);
 
   seconds = g_timer_elapsed (profiler->timer, NULL);
 
@@ -194,7 +197,7 @@ ephy_profiler_dump (EphyProfiler *profiler)
 static void
 ephy_profiler_free (EphyProfiler *profiler)
 {
-  g_return_if_fail (profiler != NULL);
+  g_assert (profiler != NULL);
 
   g_timer_destroy (profiler->timer);
   g_free (profiler->name);
@@ -210,7 +213,8 @@ ephy_profiler_free (EphyProfiler *profiler)
  * Starts a new profiler on @module naming it @name.
  **/
 void
-ephy_profiler_start (const char *name, const char *module)
+ephy_profiler_start (const char *name,
+                     const char *module)
 {
   EphyProfiler *profiler;
 
@@ -221,7 +225,8 @@ ephy_profiler_start (const char *name, const char *module)
   }
 
   if (!ephy_profile_all_modules &&
-      (ephy_profile_modules == NULL || !ephy_should_profile (module))) return;
+      (ephy_profile_modules == NULL || !ephy_should_profile (module)))
+    return;
 
   profiler = ephy_profiler_new (name, module);
 
@@ -240,7 +245,9 @@ ephy_profiler_stop (const char *name)
   EphyProfiler *profiler;
 
   profiler = g_hash_table_lookup (ephy_profilers_hash, name);
-  if (profiler == NULL) return;
+  if (profiler == NULL)
+    return;
+
   g_hash_table_remove (ephy_profilers_hash, name);
 
   ephy_profiler_dump (profiler);
