@@ -299,8 +299,8 @@ char *
 ephy_default_profile_dir (void)
 {
   return profile_dir_type == EPHY_PROFILE_DIR_TEST ?
-    g_strdup (ephy_profile_dir ()) :
-    g_build_filename (g_get_user_data_dir (), "epiphany", NULL);
+         g_strdup (ephy_profile_dir ()) :
+         g_build_filename (g_get_user_data_dir (), "epiphany", NULL);
 }
 
 /**
@@ -314,8 +314,8 @@ char *
 ephy_default_cache_dir (void)
 {
   return profile_dir_type == EPHY_PROFILE_DIR_TEST ?
-    g_build_filename (ephy_profile_dir (), "cache", NULL) :
-    g_build_filename (g_get_user_cache_dir (), "epiphany", NULL);
+         g_build_filename (ephy_profile_dir (), "cache", NULL) :
+         g_build_filename (g_get_user_cache_dir (), "epiphany", NULL);
 }
 
 /**
@@ -329,8 +329,8 @@ char *
 ephy_default_config_dir (void)
 {
   return profile_dir_type == EPHY_PROFILE_DIR_TEST ?
-    g_build_filename (ephy_profile_dir (), "config", NULL) :
-    g_build_filename (g_get_user_config_dir (), "epiphany", NULL);
+         g_build_filename (ephy_profile_dir (), "config", NULL) :
+         g_build_filename (g_get_user_config_dir (), "epiphany", NULL);
 }
 
 /**
@@ -345,13 +345,14 @@ ephy_default_config_dir (void)
  * Returns: %FALSE if the profile dir couldn't be created or accessed
  **/
 gboolean
-ephy_file_helpers_init (const char          *profile_dir,
-                        EphyFileHelpersFlags flags,
-                        GError             **error)
+ephy_file_helpers_init (const char            *profile_dir,
+                        EphyFileHelpersFlags   flags,
+                        GError               **error)
 {
   gboolean ret = TRUE;
   gboolean private_profile;
   gboolean steal_data_from_profile;
+  g_autofree char *app_file = NULL;
 
   ephy_file_helpers_error_quark = g_quark_from_static_string ("ephy-file-helpers-error");
 
@@ -373,7 +374,7 @@ ephy_file_helpers_init (const char          *profile_dir,
       g_object_unref (file);
     }
 
-    g_autofree char *app_file = g_build_filename (profile_dir, ".app", NULL);
+    app_file = g_build_filename (profile_dir, ".app", NULL);
     if (g_file_test (app_file, G_FILE_TEST_EXISTS)) {
       const char *app_name = ephy_web_application_get_program_name_from_profile_directory (profile_dir_global);
       cache_dir = g_build_filename (g_get_user_cache_dir (), app_name, NULL);
@@ -417,6 +418,7 @@ ephy_file_helpers_init (const char          *profile_dir,
     ret = ephy_ensure_dir_exists (ephy_profile_dir (), error);
     ephy_ensure_dir_exists (ephy_cache_dir (), NULL);
     ephy_ensure_dir_exists (ephy_config_dir (), NULL);
+    ephy_ensure_dir_exists (ephy_file_tmp_dir (), NULL);
   }
 
   if (steal_data_from_profile && profile_dir) {
@@ -501,8 +503,8 @@ ephy_file_helpers_shutdown (void)
  * Returns: %TRUE if @dir exists and is a directory
  **/
 gboolean
-ephy_ensure_dir_exists (const char *dir,
-                        GError    **error)
+ephy_ensure_dir_exists (const char  *dir,
+                        GError     **error)
 {
   if (g_file_test (dir, G_FILE_TEST_EXISTS) &&
       !g_file_test (dir, G_FILE_TEST_IS_DIR)) {
@@ -539,12 +541,12 @@ ephy_ensure_dir_exists (const char *dir,
 }
 
 static gboolean
-launch_application (GAppInfo  *app,
-                    GList     *files,
-                    guint32    user_time)
+launch_application (GAppInfo *app,
+                    GList    *files,
+                    guint32   user_time)
 {
-  g_autoptr(GdkAppLaunchContext) context = NULL;
-  g_autoptr(GError) error = NULL;
+  g_autoptr (GdkAppLaunchContext) context = NULL;
+  g_autoptr (GError) error = NULL;
   GdkDisplay *display;
   GdkScreen *screen;
   gboolean res;
@@ -583,7 +585,7 @@ ephy_file_launch_desktop_file (const char                   *filename,
                                guint32                       user_time,
                                EphyFileHelpersNotFlatpakTag  tag)
 {
-  g_autoptr(GDesktopAppInfo) app = NULL;
+  g_autoptr (GDesktopAppInfo) app = NULL;
 
   /* This is impossible to implement inside flatpak. Higher layers must
    * ensure we don't get here.
@@ -602,7 +604,7 @@ launch_via_uri_handler (GFile *file)
   const char *uri;
   GdkDisplay *display;
   GdkAppLaunchContext *context;
-  g_autoptr(GError) error = NULL;
+  g_autoptr (GError) error = NULL;
 
   display = gdk_display_get_default ();
   context = gdk_display_get_app_launch_context (display);
@@ -634,8 +636,8 @@ ephy_file_launch_handler (GFile   *file,
 {
   GAppInfo *app = NULL;
   gboolean ret = FALSE;
-  g_autoptr(GList) list = NULL;
-  g_autoptr(GError) error = NULL;
+  g_autoptr (GList) list = NULL;
+  g_autoptr (GError) error = NULL;
 
   g_assert (file != NULL);
 
@@ -666,9 +668,9 @@ open_in_default_handler (const char                   *uri,
                          GdkScreen                    *screen,
                          EphyFileHelpersNotFlatpakTag  tag)
 {
-  g_autoptr(GdkAppLaunchContext) context = NULL;
-  g_autoptr(GAppInfo) appinfo = NULL;
-  g_autoptr(GError) error = NULL;
+  g_autoptr (GdkAppLaunchContext) context = NULL;
+  g_autoptr (GAppInfo) appinfo = NULL;
+  g_autoptr (GError) error = NULL;
   GList uris;
 
   /* This is impossible to implement inside flatpak. Higher layers must
@@ -682,6 +684,11 @@ open_in_default_handler (const char                   *uri,
   gdk_app_launch_context_set_timestamp (context, timestamp);
 
   appinfo = g_app_info_get_default_for_type (mime_type, TRUE);
+  if (!appinfo) {
+    g_warning ("Failed to get default app for MIME type: %s", mime_type);
+    return FALSE;
+  }
+
   uris.data = (gpointer)uri;
   uris.next = uris.prev = NULL;
 
@@ -706,7 +713,6 @@ ephy_file_open_uri_in_default_browser (const char                   *uri,
  * ephy_file_browse_to:
  * @file: a #GFile
  * @user_time: user_time to prevent focus stealing
- * @tag: used to guard against improper usage
  *
  * Launches the default application for browsing directories to point to
  * @file. E.g. nautilus will jump to @file within its directory and
@@ -715,13 +721,17 @@ ephy_file_open_uri_in_default_browser (const char                   *uri,
  * Returns: %TRUE if the launch succeeded
  **/
 gboolean
-ephy_file_browse_to (GFile                        *file,
-                     guint32                       user_time,
-                     EphyFileHelpersNotFlatpakTag  tag)
+ephy_file_browse_to (GFile   *file,
+                     guint32  user_time)
 {
   g_autofree char *uri = g_file_get_uri (file);
 
-  return open_in_default_handler (uri, "inode/directory", user_time, NULL, tag);
+  if (ephy_is_running_inside_flatpak ()) {
+    ephy_open_directory_via_flatpak_portal (uri);
+    return TRUE;
+  }
+
+  return open_in_default_handler (uri, "inode/directory", user_time, NULL, EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
 }
 
 /**
@@ -734,7 +744,8 @@ ephy_file_browse_to (GFile                        *file,
  * Returns: %TRUE if delete succeeded
  **/
 gboolean
-ephy_file_delete_dir_recursively (const char *directory, GError **error)
+ephy_file_delete_dir_recursively (const char  *directory,
+                                  GError     **error)
 {
   GDir *dir;
   const char *file_name;
