@@ -139,11 +139,13 @@ ephy_notification_init (EphyNotification *self)
   gtk_container_add (GTK_CONTAINER (self), self->grid);
 
   self->head = gtk_label_new (NULL);
+  gtk_label_set_line_wrap (GTK_LABEL (self->head), TRUE);
   gtk_widget_set_halign (self->head, GTK_ALIGN_CENTER);
   gtk_widget_set_hexpand (self->head, TRUE);
   gtk_grid_attach (GTK_GRID (self->grid), self->head, 0, 0, 1, 1);
 
   self->body = gtk_label_new (NULL);
+  gtk_label_set_line_wrap (GTK_LABEL (self->body), TRUE);
   gtk_widget_set_halign (self->body, GTK_ALIGN_CENTER);
   gtk_widget_set_hexpand (self->body, TRUE);
   gtk_grid_attach (GTK_GRID (self->grid), self->body, 0, 1, 1, 1);
@@ -211,3 +213,10 @@ ephy_notification_show (EphyNotification *self)
   ephy_notification_container_add_notification (ephy_notification_container_get_default (),
                                                 GTK_WIDGET (self));
 }
+
+gboolean
+ephy_notification_is_duplicate (EphyNotification *notification_a,
+                                EphyNotification *notification_b)
+{
+  return g_strcmp0 (notification_a->head_msg, notification_b->head_msg) == 0 && g_strcmp0 (notification_a->body_msg, notification_b->body_msg) == 0;
+}
