@@ -167,6 +167,8 @@ permission_type_to_string (EphyPermissionType type)
       return "video-device-permission";
     case EPHY_PERMISSION_TYPE_SHOW_ADS:
       return "advertisement-permission";
+    case EPHY_PERMISSION_TYPE_AUTOPLAY_POLICY:
+      return "autoplay-permission";
     default:
       g_assert_not_reached ();
   }
@@ -177,7 +179,11 @@ ephy_permissions_manager_get_permission (EphyPermissionsManager *manager,
                                          EphyPermissionType      type,
                                          const char             *origin)
 {
-  GSettings *settings = ephy_permissions_manager_get_settings_for_origin (manager, origin);
+  GSettings *settings;
+
+  g_assert (type != EPHY_PERMISSION_TYPE_ACCESS_WEBCAM_AND_MICROPHONE);
+
+  settings = ephy_permissions_manager_get_settings_for_origin (manager, origin);
   return g_settings_get_enum (settings, permission_type_to_string (type));
 }
 
@@ -245,6 +251,8 @@ ephy_permissions_manager_set_permission (EphyPermissionsManager *manager,
   WebKitSecurityOrigin *webkit_origin;
   GSettings *settings;
 
+  g_assert (type != EPHY_PERMISSION_TYPE_ACCESS_WEBCAM_AND_MICROPHONE);
+
   webkit_origin = webkit_security_origin_new_for_uri (origin);
   if (webkit_origin == NULL)
     return;
