@@ -220,6 +220,9 @@ ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface)
   orig_width = cairo_image_surface_get_width (surface);
   orig_height = cairo_image_surface_get_height (surface);
 
+  if (!orig_width || !orig_height)
+    return NULL;
+
   if (orig_width < EPHY_THUMBNAIL_WIDTH ||
       orig_height < EPHY_THUMBNAIL_HEIGHT) {
     snapshot = gdk_pixbuf_get_from_surface (surface,
@@ -230,10 +233,13 @@ ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface)
                                       EPHY_THUMBNAIL_HEIGHT,
                                       GDK_INTERP_TILES);
   } else {
+    gfloat width_ratio = (gfloat)EPHY_THUMBNAIL_WIDTH / (gfloat)orig_width;
+    gfloat new_height = orig_height * width_ratio;
+
     snapshot = gdk_pixbuf_get_from_surface (surface, 0, 0, orig_width, orig_height);
     scaled = gdk_pixbuf_scale_simple (snapshot,
                                       EPHY_THUMBNAIL_WIDTH,
-                                      EPHY_THUMBNAIL_HEIGHT,
+                                      new_height,
                                       GDK_INTERP_BILINEAR);
   }
 
@@ -393,6 +399,14 @@ save_snapshot (cairo_surface_t *surface,
   SnapshotAsyncData *data = g_task_get_task_data (task);
 
   data->snapshot = ephy_snapshot_service_prepare_snapshot (surface);
+  if (!data->snapshot) {
+    g_task_return_new_error (task,
+                             EPHY_SNAPSHOT_SERVICE_ERROR,
+                             EPHY_SNAPSHOT_SERVICE_ERROR_WEB_VIEW,
+                             "WebView returned invalid snapshot for \"%s\"", data->url);
+    g_object_unref (task);
+    return;
+  }
 
   ephy_snapshot_service_save_snapshot_async (g_task_get_source_object (task),
                                              data->snapshot,
@@ -437,7 +451,7 @@ retrieve_snapshot_from_web_view (GTask *task)
   }
 
   webkit_web_view_get_snapshot (data->web_view,
-                                WEBKIT_SNAPSHOT_REGION_VISIBLE,
+                                WEBKIT_SNAPSHOT_REGION_FULL_DOCUMENT,
                                 WEBKIT_SNAPSHOT_OPTIONS_NONE,
                                 g_task_get_cancellable (task),
                                 (GAsyncReadyCallback)on_snapshot_ready,
