@@ -40,14 +40,14 @@
  * <normalized-name>-checksum.
  *
  * The id is used to uniquely identify the app.
- *  - Program name: epiphany-<id>
+ *  - Program name: org.gnome.Epiphany.WebApp-<id>
  *  - Profile directory: <program-name>
  *  - Desktop file: <profile-dir>/<program-name>.desktop
  *
  * System web applications have a profile dir without a desktop file.
  */
 
-#define EPHY_WEB_APP_PROGRAM_NAME_PREFIX "epiphany-"
+#define EPHY_WEB_APP_PROGRAM_NAME_PREFIX "org.gnome.Epiphany.WebApp-"
 
 char *
 ephy_web_application_get_app_id_from_name (const char *name)
@@ -363,16 +363,18 @@ create_desktop_file (const char *id,
  * @address: the address of the new web application
  * @name: the name for the new web application
  * @icon: the icon for the new web application
+ * @options: the options for the new web application
  *
  * Creates a new Web Application for @address.
  *
  * Returns: (transfer-full): the path to the desktop file representing the new application
  **/
 char *
-ephy_web_application_create (const char *id,
-                             const char *address,
-                             const char *name,
-                             GdkPixbuf  *icon)
+ephy_web_application_create (const char                *id,
+                             const char                *address,
+                             const char                *name,
+                             GdkPixbuf                 *icon,
+                             EphyWebApplicationOptions  options)
 {
   g_autofree char *app_file = NULL;
   g_autofree char *profile_dir = NULL;
@@ -408,7 +410,7 @@ ephy_web_application_create (const char *id,
   /* Create the deskop file. */
   desktop_file_path = create_desktop_file (id, name, address, profile_dir, icon);
   if (desktop_file_path)
-    ephy_web_application_initialize_settings (profile_dir);
+    ephy_web_application_initialize_settings (profile_dir, options);
 
   return g_steal_pointer (&desktop_file_path);
 }
@@ -416,22 +418,59 @@ ephy_web_application_create (const char *id,
 char *
 ephy_web_application_ensure_for_app_info (GAppInfo *app_info)
 {
-  char *id;
-  char *profile_dir;
+  g_autofree char *id = NULL;
+  g_autofree char *profile_dir = NULL;
+  g_autofree char *app_file = NULL;
+  int fd;
 
   id = ephy_web_application_get_app_id_from_name (g_app_info_get_name (app_info));
   profile_dir = ephy_web_application_get_profile_directory (id);
-  g_free (id);
 
+  /* Create the profile directory, populate it. */
   if (g_mkdir (profile_dir, 488) == -1) {
     if (errno == EEXIST)
-      return profile_dir;
+      return g_steal_pointer (&profile_dir);
 
-    g_free (profile_dir);
     return NULL;
   }
 
-  return profile_dir;
+  /* Skip migration for new web apps. */
+  ephy_profile_utils_set_migration_version_for_profile_dir (EPHY_PROFILE_MIGRATION_VERSION, profile_dir);
+
+  /* Create an .app file. */
+  app_file = g_build_filename (profile_dir, ".app", NULL);
+  fd = g_open (app_file, O_WRONLY | O_CREAT | O_TRUNC, 0644);
+  if (fd < 0) {
+    g_warning ("Failed to create .app file: %s", g_strerror (errno));
+    return NULL;
+  }
+  close (fd);
+
+  /* Create the deskop file. */
+  if (G_IS_DESKTOP_APP_INFO (app_info)) {
+    const char *source_name = NULL;
+    g_autofree char *dest_name = NULL;
+    g_autofree char *desktop_basename = NULL;
+    g_autoptr (GFile) source = NULL;
+    g_autoptr (GFile) dest = NULL;
+    g_autoptr (GError) error = NULL;
+
+    source_name = g_desktop_app_info_get_filename (G_DESKTOP_APP_INFO (app_info));
+    source = g_file_new_for_path (source_name);
+
+    desktop_basename = get_app_desktop_filename (id);
+    dest_name = g_build_filename (profile_dir, desktop_basename, NULL);
+    dest = g_file_new_for_path (dest_name);
+
+    g_file_copy (source, dest, G_FILE_COPY_NONE, NULL, NULL, NULL, &error);
+
+    if (error)
+      g_warning ("Couldn't copy desktop file: %s", error->message);
+
+    ephy_web_application_initialize_settings (profile_dir, EPHY_WEB_APPLICATION_NONE);
+  }
+
+  return g_steal_pointer (&profile_dir);
 }
 
 void
@@ -711,7 +750,8 @@ ephy_web_application_exists (const char *id)
 }
 
 void
-ephy_web_application_initialize_settings (const char *profile_directory)
+ephy_web_application_initialize_settings (const char                *profile_directory,
+                                          EphyWebApplicationOptions  options)
 {
   GSettings *settings;
   GSettings *web_app_settings;
@@ -752,6 +792,16 @@ ephy_web_application_initialize_settings (const char *profile_directory)
 
   g_object_unref (settings);
   g_object_unref (web_app_settings);
+
+  if (options & EPHY_WEB_APPLICATION_MOBILE_CAPABLE) {
+    path = g_build_path ("/", "/org/gnome/epiphany/web-apps/", name, "webapp/", NULL);
+    web_app_settings = g_settings_new_with_path (EPHY_PREFS_WEB_APP_SCHEMA, path);
+    g_free (path);
+
+    g_settings_set_boolean (web_app_settings, EPHY_PREFS_WEB_APP_MOBILE_CAPABLE, TRUE);
+    g_object_unref (web_app_settings);
+  }
+
   g_free (name);
 }
 
