@@ -28,11 +28,11 @@
 
 #define EXPIRATION_THRESHOLD (8 * 60 * 60)
 
-/* Keep this lower than 200 or else you'll get "too many SQL variables" error
- * in ephy_gsb_storage_insert_batch(). SQLITE_MAX_VARIABLE_NUMBER is hardcoded
- * in sqlite3 as 999.
+/* Keep this lower than 6533 (SQLITE_MAX_VARIABLE_NUMBER / 5 slots) or else
+ * you'll get "too many SQL variables" error in ephy_gsb_storage_insert_batch().
+ * SQLITE_MAX_VARIABLE_NUMBER is hardcoded in sqlite3 (>= 3.22) as 32766.
  */
-#define BATCH_SIZE 199
+#define BATCH_SIZE 6553
 
 /* Increment schema version if you:
  * 1) Modify the database table structure.
@@ -923,11 +923,11 @@ ephy_gsb_storage_get_hash_prefixes_to_delete (EphyGSBStorage    *self,
   g_assert (list);
   g_assert (indices);
 
+  *num_prefixes = 0;
+
   if (!self->is_operable)
     return NULL;
 
-  *num_prefixes = 0;
-
   sql = "SELECT value FROM hash_prefix WHERE "
         "threat_type=? AND platform_type=? AND threat_entry_type=? "
         "ORDER BY value";
@@ -1064,7 +1064,7 @@ ephy_gsb_storage_delete_hash_prefixes_internal (EphyGSBStorage    *self,
   GList *prefixes = NULL;
   GList *head = NULL;
   GHashTable *set;
-  gsize num_prefixes;
+  gsize num_prefixes = 0;
 
   g_assert (EPHY_IS_GSB_STORAGE (self));
   g_assert (list);
