@@ -278,6 +278,7 @@ ephy_sync_debug_get_signed_certificate (const char           *session_token,
   char *n;
   char *e;
   guint status_code;
+  g_autofree char *accounts_server = NULL;
 
   g_assert (session_token);
   g_assert (keypair);
@@ -298,7 +299,8 @@ ephy_sync_debug_get_signed_certificate (const char           *session_token,
   json_node_set_object (node, json_body);
   body = json_to_string (node, FALSE);
 
-  url = g_strdup_printf ("%s/certificate/sign", EPHY_SYNC_FX_ACCOUNTS_SERVER_URL);
+  accounts_server = ephy_sync_utils_get_accounts_server ();
+  url = g_strdup_printf ("%s/certificate/sign", accounts_server);
   msg = ephy_sync_debug_prepare_soup_message (url, "POST", body,
                                               id_hex, key, 32);
   session = soup_session_new ();
@@ -359,6 +361,7 @@ ephy_sync_debug_get_storage_credentials (char **storage_endpoint,
   const char *session_token;
   guint status_code;
   gboolean success = FALSE;
+  g_autofree char *token_server = NULL;
 
   secrets = ephy_sync_debug_load_secrets ();
   if (!secrets)
@@ -370,13 +373,14 @@ ephy_sync_debug_get_storage_credentials (char **storage_endpoint,
   if (!certificate)
     goto free_keypair;
 
-  audience = ephy_sync_utils_get_audience (EPHY_SYNC_FX_TOKEN_SERVER_URL);
+  token_server = ephy_sync_utils_get_token_server ();
+  audience = ephy_sync_utils_get_audience (token_server);
   assertion = ephy_sync_crypto_create_assertion (certificate, audience, 300, keypair);
   kb = ephy_sync_utils_decode_hex (json_object_get_string_member (secrets, "master_key"));
   hashed_kb = g_compute_checksum_for_data (G_CHECKSUM_SHA256, kb, 32);
   client_state = g_strndup (hashed_kb, 32);
   authorization = g_strdup_printf ("BrowserID %s", assertion);
-  msg = soup_message_new ("GET", EPHY_SYNC_FX_TOKEN_SERVER_URL);
+  msg = soup_message_new ("GET", token_server);
   soup_message_headers_append (msg->request_headers, "X-Client-State", client_state);
   soup_message_headers_append (msg->request_headers, "authorization", authorization);
   session = soup_session_new ();
@@ -1024,6 +1028,7 @@ ephy_sync_debug_view_connected_devices (void)
   char *id_hex;
   char *url;
   const char *session_token;
+  g_autofree char *accounts_server = NULL;
 
   secrets = ephy_sync_debug_load_secrets ();
   if (!secrets)
@@ -1032,7 +1037,8 @@ ephy_sync_debug_view_connected_devices (void)
   session_token = json_object_get_string_member (secrets, "session_token");
   ephy_sync_crypto_derive_session_token (session_token, &id, &key, &tmp);
 
-  url = g_strdup_printf ("%s/account/devices", EPHY_SYNC_FX_ACCOUNTS_SERVER_URL);
+  accounts_server = ephy_sync_utils_get_accounts_server ();
+  url = g_strdup_printf ("%s/account/devices", accounts_server);
   id_hex = ephy_sync_utils_encode_hex (id, 32);
   msg = ephy_sync_debug_prepare_soup_message (url, "GET", NULL, id_hex, key, 32);
   session = soup_session_new ();
@@ -1076,6 +1082,7 @@ ephy_sync_debug_get_current_device (void)
   char *url;
   const char *session_token;
   guint status_code;
+  g_autofree char *accounts_server = NULL;
 
   secrets = ephy_sync_debug_load_secrets ();
   if (!secrets)
@@ -1084,7 +1091,8 @@ ephy_sync_debug_get_current_device (void)
   session_token = json_object_get_string_member (secrets, "session_token");
   ephy_sync_crypto_derive_session_token (session_token, &id, &key, &tmp);
 
-  url = g_strdup_printf ("%s/account/devices", EPHY_SYNC_FX_ACCOUNTS_SERVER_URL);
+  accounts_server = ephy_sync_utils_get_accounts_server ();
+  url = g_strdup_printf ("%s/account/devices", accounts_server);
   id_hex = ephy_sync_utils_encode_hex (id, 32);
   msg = ephy_sync_debug_prepare_soup_message (url, "GET", NULL, id_hex, key, 32);
   session = soup_session_new ();
