@@ -66,8 +66,20 @@ void                 ephy_password_manager_query                    (EphyPasswor
                                                                      const char                       *password_field,
                                                                      EphyPasswordManagerQueryCallback  callback,
                                                                      gpointer                          user_data);
+gboolean             ephy_password_manager_find                     (EphyPasswordManager              *self,
+                                                                     const char                       *origin,
+                                                                     const char                       *target_origin,
+                                                                     const char                       *username,
+                                                                     const char                       *username_field,
+                                                                     const char                       *password_field);
 void                 ephy_password_manager_forget                    (EphyPasswordManager *self,
-                                                                      const char          *id);
+                                                                      const char          *id,
+                                                                      GCancellable        *cancellable,
+                                                                      GAsyncReadyCallback  callback,
+                                                                      gpointer             user_data);
+gboolean             ephy_password_manager_forget_finish             (EphyPasswordManager  *self,
+                                                                      GAsyncResult         *result,
+                                                                      GError              **error);
 void                 ephy_password_manager_forget_all                (EphyPasswordManager *self);
 void                 ephy_password_manager_export_to_js_context      (EphyPasswordManager *self,
                                                                       JSCContext          *js_context,
