@@ -45,8 +45,6 @@ gboolean        ephy_location_entry_get_can_undo               (EphyLocationEntr
 
 gboolean        ephy_location_entry_get_can_redo               (EphyLocationEntry *entry);
 
-GSList         *ephy_location_entry_get_search_terms           (EphyLocationEntry *entry);
-
 gboolean        ephy_location_entry_reset                      (EphyLocationEntry *entry);
 
 void            ephy_location_entry_undo_reset                 (EphyLocationEntry *entry);
@@ -82,4 +80,7 @@ void            ephy_location_entry_set_progress               (EphyLocationEntr
                                                                 gdouble            progress,
                                                                 gboolean           loading);
 
+void            ephy_location_entry_set_mobile_popdown         (EphyLocationEntry *entry,
+                                                                gboolean           mobile_popdown);
+
 G_END_DECLS
