@@ -65,6 +65,7 @@ struct _EphySecurityPopover {
   GtkWidget *access_location_combobox;
   GtkWidget *access_microphone_combobox;
   GtkWidget *access_webcam_combobox;
+  GtkWidget *autoplay_combobox;
   GtkWidget *grid;
   GTlsCertificate *certificate;
   GTlsCertificateFlags tls_errors;
@@ -155,6 +156,7 @@ ephy_security_popover_set_address (EphySecurityPopover *popover,
   set_permission_combobox_state (permissions_manager, EPHY_PERMISSION_TYPE_ACCESS_LOCATION, origin, popover->access_location_combobox);
   set_permission_combobox_state (permissions_manager, EPHY_PERMISSION_TYPE_ACCESS_MICROPHONE, origin, popover->access_microphone_combobox);
   set_permission_combobox_state (permissions_manager, EPHY_PERMISSION_TYPE_ACCESS_WEBCAM, origin, popover->access_webcam_combobox);
+  set_permission_combobox_state (permissions_manager, EPHY_PERMISSION_TYPE_AUTOPLAY_POLICY, origin, popover->autoplay_combobox);
 }
 
 static void
@@ -482,11 +484,20 @@ on_access_webcam_combobox_changed (GtkComboBox         *box,
   handle_permission_combobox_changed (popover, gtk_combo_box_get_active (box), EPHY_PERMISSION_TYPE_ACCESS_WEBCAM);
 }
 
+static void
+on_autoplay_policy_combobox_changed (GtkComboBox         *box,
+                                     EphySecurityPopover *popover)
+{
+  handle_permission_combobox_changed (popover, gtk_combo_box_get_active (box), EPHY_PERMISSION_TYPE_AUTOPLAY_POLICY);
+}
+
 static GtkWidget *
 add_permission_combobox (EphySecurityPopover *popover,
                          const gchar         *name,
                          gpointer             callback,
-                         gboolean             no_ask)
+                         GtkSizeGroup        *size_group,
+                         gboolean             no_ask,
+                         const gchar         *third_option_name)
 {
   GtkWidget *widget;
   GtkWidget *hbox;
@@ -500,14 +511,17 @@ add_permission_combobox (EphySecurityPopover *popover,
   gtk_box_pack_start (GTK_BOX (hbox), tmp, TRUE, TRUE, 6);
 
   widget = gtk_combo_box_text_new ();
-  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget), _("Yes"));
-  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget), _("No"));
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget), _("Allow"));
+  gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget), _("Deny"));
 
-  if (!no_ask)
-    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget), _("Ask"));
+  if (!no_ask) {
+    const gchar *name = third_option_name == NULL ? _("Ask") : third_option_name;
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (widget), _(name));
+  }
 
   gtk_box_pack_start (GTK_BOX (hbox), widget, FALSE, FALSE, 6);
   g_signal_connect (widget, "changed", G_CALLBACK (callback), popover);
+  gtk_size_group_add_widget (size_group, widget);
 
   return widget;
 }
@@ -517,6 +531,7 @@ ephy_security_popover_init (EphySecurityPopover *popover)
 {
   GtkWidget *permissions;
   GtkWidget *box;
+  g_autoptr (GtkSizeGroup) combo_box_size_group = NULL;
   g_autofree char *label = g_strdup_printf ("<b>%s</b>", _("Permissions"));
 
   popover->grid = gtk_grid_new ();
@@ -553,12 +568,15 @@ ephy_security_popover_init (EphySecurityPopover *popover)
   gtk_grid_attach (GTK_GRID (popover->grid), permissions, 0, 4, 2, 1);
 
   popover->permission_pos = 5;
-  popover->ad_combobox = add_permission_combobox (popover, _("Advertisements"), on_ad_combobox_changed, TRUE);
-  popover->notification_combobox = add_permission_combobox (popover, _("Notifications"), on_notification_combobox_changed, FALSE);
-  popover->save_password_combobox = add_permission_combobox (popover, _("Password saving"), on_save_password_combobox_changed, FALSE);
-  popover->access_location_combobox = add_permission_combobox (popover, _("Location access"), on_access_location_combobox_changed, FALSE);
-  popover->access_microphone_combobox = add_permission_combobox (popover, _("Microphone access"), on_access_microphone_combobox_changed, FALSE);
-  popover->access_webcam_combobox = add_permission_combobox (popover, _("Webcam access"), on_access_webcam_combobox_changed, FALSE);
+  combo_box_size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+
+  popover->ad_combobox = add_permission_combobox (popover, _("Advertisements"), on_ad_combobox_changed, combo_box_size_group, TRUE, NULL);
+  popover->notification_combobox = add_permission_combobox (popover, _("Notifications"), on_notification_combobox_changed, combo_box_size_group, FALSE, NULL);
+  popover->save_password_combobox = add_permission_combobox (popover, _("Password saving"), on_save_password_combobox_changed, combo_box_size_group, FALSE, NULL);
+  popover->access_location_combobox = add_permission_combobox (popover, _("Location access"), on_access_location_combobox_changed, combo_box_size_group, FALSE, NULL);
+  popover->access_microphone_combobox = add_permission_combobox (popover, _("Microphone access"), on_access_microphone_combobox_changed, combo_box_size_group, FALSE, NULL);
+  popover->access_webcam_combobox = add_permission_combobox (popover, _("Webcam access"), on_access_webcam_combobox_changed, combo_box_size_group, FALSE, NULL);
+  popover->autoplay_combobox = add_permission_combobox (popover, _("Media autoplay"), on_autoplay_policy_combobox_changed, combo_box_size_group, FALSE, "Without Sound");
 
   gtk_container_add (GTK_CONTAINER (popover), popover->grid);
   gtk_widget_show_all (popover->grid);
