@@ -2,23 +2,23 @@
 /*
  *  Copyright © 2014, 2015 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program. If not, see <http://www.gnu.org/licenses/>.
- *
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef EPHY_SECURITY_POPOVER_H
-#define EPHY_SECURITY_POPOVER_H
+#pragma once
 
 #include <gio/gio.h>
 #include <gtk/gtk.h>
@@ -38,5 +38,3 @@ GtkWidget *ephy_security_popover_new      (GtkWidget *relative_to,
                                            EphySecurityLevel security_level);
 
 G_END_DECLS
-
-#endif
