@@ -21,11 +21,14 @@
 
 #include "ephy-action-bar-start.h"
 
+#include "ephy-desktop-utils.h"
 #include "ephy-embed.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-utils.h"
 #include "ephy-favicon-helpers.h"
+#include "ephy-file-helpers.h"
+#include "ephy-gui.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-window.h"
@@ -43,8 +46,10 @@ struct _EphyActionBarStart {
   GtkWidget *combined_stop_reload_image;
   GtkWidget *homepage_button;
   GtkWidget *new_tab_button;
+  GtkWidget *placeholder;
 
   guint navigation_buttons_menu_timeout;
+  gboolean is_fullscreen;
 };
 
 G_DEFINE_TYPE (EphyActionBarStart, ephy_action_bar_start, GTK_TYPE_BOX)
@@ -79,7 +84,7 @@ item_enter_notify_event_cb (GtkWidget   *widget,
   text = g_object_get_data (G_OBJECT (widget), "link-message");
   ephy_web_view_set_link_message (view, text);
 
-  return FALSE;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -88,7 +93,7 @@ item_leave_notify_event_cb (GtkWidget   *widget,
                             EphyWebView *view)
 {
   ephy_web_view_set_link_message (view, NULL);
-  return FALSE;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static void
@@ -106,9 +111,16 @@ icon_loaded_cb (GObject      *source,
   }
 
   if (favicon) {
-    gtk_image_set_from_pixbuf (GTK_IMAGE (image), favicon);
+    cairo_surface_t *surface;
+
+    surface = gdk_cairo_surface_create_from_pixbuf (favicon,
+                                                    0,
+                                                    gtk_widget_get_window (GTK_WIDGET (image)));
+    gtk_image_set_from_surface (GTK_IMAGE (image), surface);
+
     gtk_widget_show (image);
 
+    cairo_surface_destroy (surface);
     g_object_unref (favicon);
   }
 
@@ -201,13 +213,13 @@ navigation_menu_item_pressed_cb (GtkWidget *menuitem,
     webkit_web_view_go_to_back_forward_list_item (web_view, item);
   }
 
-  return G_SOURCE_REMOVE;
+  return GDK_EVENT_STOP;
 }
 
 static GList *
-construct_webkit_history_list (WebKitWebView    *web_view,
-                               WebKitHistoryType hist_type,
-                               int               limit)
+construct_webkit_history_list (WebKitWebView     *web_view,
+                               WebKitHistoryType  hist_type,
+                               int                limit)
 {
   WebKitBackForwardList *back_forward_list;
 
@@ -218,8 +230,8 @@ construct_webkit_history_list (WebKitWebView    *web_view,
 }
 
 static GtkWidget *
-build_dropdown_menu (EphyWindow                    *window,
-                     EphyNavigationHistoryDirection direction)
+build_dropdown_menu (EphyWindow                     *window,
+                     EphyNavigationHistoryDirection  direction)
 {
   GtkMenuShell *menu;
   EphyEmbed *embed;
@@ -273,10 +285,10 @@ build_dropdown_menu (EphyWindow                    *window,
 }
 
 static void
-popup_history_menu (GtkWidget                     *widget,
-                    EphyWindow                    *window,
-                    EphyNavigationHistoryDirection direction,
-                    GdkEventButton                *event)
+popup_history_menu (GtkWidget                      *widget,
+                    EphyWindow                     *window,
+                    EphyNavigationHistoryDirection  direction,
+                    GdkEventButton                 *event)
 {
   GtkWidget *menu;
 
@@ -304,33 +316,42 @@ navigation_button_press_event_cb (GtkButton *button,
 {
   EphyActionBarStart *action_bar_start = EPHY_ACTION_BAR_START (user_data);
   EphyNavigationHistoryDirection direction;
-  const gchar *action_name;
+  PopupData *data;
+  gboolean is_back = FALSE;
 
-  action_name = gtk_actionable_get_action_name (GTK_ACTIONABLE (button));
+  is_back = (GTK_WIDGET (button) == action_bar_start->navigation_back);
 
-  direction = strstr (action_name, "back") ? EPHY_NAVIGATION_HISTORY_DIRECTION_BACK
-                                           : EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD;
+  direction = is_back ? EPHY_NAVIGATION_HISTORY_DIRECTION_BACK
+                      : EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD;
 
-  if (((GdkEventButton *)event)->button == GDK_BUTTON_SECONDARY) {
-    popup_history_menu (GTK_WIDGET (button), EPHY_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (action_bar_start), EPHY_TYPE_WINDOW)),
-                        direction, (GdkEventButton *)event);
-  } else {
-    PopupData *data;
-
-    data = g_new (PopupData, 1);
-    data->button = GTK_WIDGET (button);
-    data->window = EPHY_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (action_bar_start), EPHY_TYPE_WINDOW));
-    data->direction = direction;
-    data->event = (GdkEventButton *)event;
-
-    action_bar_start->navigation_buttons_menu_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT, 500,
-                                                                      (GSourceFunc)menu_timeout_cb,
-                                                                      data,
-                                                                      (GDestroyNotify)g_free);
-    g_source_set_name_by_id (action_bar_start->navigation_buttons_menu_timeout, "[epiphany] menu_timeout_cb");
+  switch (((GdkEventButton *)event)->button) {
+    case GDK_BUTTON_SECONDARY:
+      popup_history_menu (GTK_WIDGET (button), EPHY_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (action_bar_start), EPHY_TYPE_WINDOW)),
+                          direction, (GdkEventButton *)event);
+      return GDK_EVENT_STOP;
+    case GDK_BUTTON_MIDDLE:
+      /* If a desktop-wide middle-click titlebar action is enabled, we want to
+       * prevent that action from occurring because we handle middle click in
+       * navigation_button_release_event_cb().
+       */
+      return GDK_EVENT_STOP;
+    default:
+      break;
   }
 
-  return FALSE;
+  data = g_new (PopupData, 1);
+  data->button = GTK_WIDGET (button);
+  data->window = EPHY_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (action_bar_start), EPHY_TYPE_WINDOW));
+  data->direction = direction;
+  data->event = (GdkEventButton *)event;
+
+  action_bar_start->navigation_buttons_menu_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT, 500,
+                                                                          (GSourceFunc)menu_timeout_cb,
+                                                                          data,
+                                                                          (GDestroyNotify)g_free);
+  g_source_set_name_by_id (action_bar_start->navigation_buttons_menu_timeout, "[epiphany] menu_timeout_cb");
+
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -342,40 +363,52 @@ navigation_button_release_event_cb (GtkButton *button,
   GActionGroup *action_group;
   GAction *action;
   EphyNavigationHistoryDirection direction;
-  const gchar *action_name;
+  gboolean is_back = FALSE;
+  gboolean open_in_new_tab = FALSE;
+  gboolean open_in_current_tab = FALSE;
+  GdkEventType type = GDK_NOTHING;
+  guint state = 0, button_val = (guint) - 1, keyval = (guint) - 1;
 
-  if (action_bar_start->navigation_buttons_menu_timeout > 0) {
-    g_source_remove (action_bar_start->navigation_buttons_menu_timeout);
-    action_bar_start->navigation_buttons_menu_timeout = 0;
-  }
+  ephy_gui_get_current_event (&type, &state, &button_val, &keyval);
+  is_back = (GTK_WIDGET (button) == action_bar_start->navigation_back);
+
+  g_clear_handle_id (&action_bar_start->navigation_buttons_menu_timeout, g_source_remove);
 
-  action_name = gtk_actionable_get_action_name (GTK_ACTIONABLE (button));
   action_group = gtk_widget_get_action_group (gtk_widget_get_ancestor (GTK_WIDGET (action_bar_start), EPHY_TYPE_WINDOW), "toolbar");
 
-  direction = strcmp (action_name, "toolbar.navigation-back") == 0 ? EPHY_NAVIGATION_HISTORY_DIRECTION_BACK
-                                                                   : EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD;
+  direction = is_back ? EPHY_NAVIGATION_HISTORY_DIRECTION_BACK
+                      : EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD;
 
-  switch (((GdkEventButton *)event)->button) {
-    case GDK_BUTTON_MIDDLE:
-      if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_BACK) {
-        action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
-                                             "navigation-back-new-tab");
-        g_action_activate (action, NULL);
-      } else if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD) {
-        action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
-                                             "navigation-forward-new-tab");
-        g_action_activate (action, NULL);
-      }
-      break;
-    case GDK_BUTTON_SECONDARY:
-      popup_history_menu (GTK_WIDGET (button), EPHY_WINDOW (gtk_widget_get_ancestor (GTK_WIDGET (action_bar_start), EPHY_TYPE_WINDOW)),
-                          direction, (GdkEventButton *)event);
-      break;
-    default:
-      break;
+  open_in_new_tab = (((GdkEventButton *)event)->button == GDK_BUTTON_MIDDLE) || (state == GDK_CONTROL_MASK);
+  open_in_current_tab = ((GdkEventButton *)event)->button == GDK_BUTTON_PRIMARY;
+
+  if (open_in_new_tab) {
+    if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_BACK) {
+      action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
+                                           "navigation-back-new-tab");
+      g_action_activate (action, NULL);
+    } else if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD) {
+      action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
+                                           "navigation-forward-new-tab");
+      g_action_activate (action, NULL);
+    }
+    return GDK_EVENT_STOP;
   }
 
-  return G_SOURCE_REMOVE;
+  if (open_in_current_tab) {
+    if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_BACK) {
+      action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
+                                           "navigation-back");
+      g_action_activate (action, NULL);
+    } else if (direction == EPHY_NAVIGATION_HISTORY_DIRECTION_FORWARD) {
+      action = g_action_map_lookup_action (G_ACTION_MAP (action_group),
+                                           "navigation-forward");
+      g_action_activate (action, NULL);
+    }
+    return GDK_EVENT_STOP;
+  }
+
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -393,12 +426,12 @@ homepage_button_release_event_cb (GtkButton *button,
     case GDK_BUTTON_MIDDLE:
       action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "homepage-new-tab");
       g_action_activate (action, NULL);
-      break;
+      return GDK_EVENT_STOP;
     default:
       break;
   }
 
-  return G_SOURCE_REMOVE;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -416,12 +449,12 @@ new_tab_button_release_event_cb (GtkButton *button,
     case GDK_BUTTON_MIDDLE:
       action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "new-tab-from-clipboard");
       g_action_activate (action, NULL);
-      break;
+      return GDK_EVENT_STOP;
     default:
       break;
   }
 
-  return G_SOURCE_REMOVE;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -431,12 +464,9 @@ navigation_leave_notify_event_cb (GtkButton *button,
 {
   EphyActionBarStart *action_bar_start = EPHY_ACTION_BAR_START (user_data);
 
-  if (action_bar_start->navigation_buttons_menu_timeout > 0) {
-    g_source_remove (action_bar_start->navigation_buttons_menu_timeout);
-    action_bar_start->navigation_buttons_menu_timeout = 0;
-  }
+  g_clear_handle_id (&action_bar_start->navigation_buttons_menu_timeout, g_source_remove);
 
-  return G_SOURCE_REMOVE;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static void
@@ -448,7 +478,10 @@ homepage_url_changed (GSettings  *settings,
   gboolean show_button;
 
   setting = g_settings_get_string (settings, key);
-  show_button = setting && setting[0] && g_strcmp0 (setting, "about:blank") != 0;
+  if (setting && setting[0])
+    show_button = g_strcmp0 (setting, "about:blank") != 0;
+  else
+    show_button = is_desktop_pantheon ();
 
   gtk_widget_set_visible (button, show_button);
   g_free (setting);
@@ -459,20 +492,32 @@ ephy_action_bar_start_dispose (GObject *object)
 {
   EphyActionBarStart *action_bar_start = EPHY_ACTION_BAR_START (object);
 
-  if (action_bar_start->navigation_buttons_menu_timeout > 0) {
-    g_source_remove (action_bar_start->navigation_buttons_menu_timeout);
-    action_bar_start->navigation_buttons_menu_timeout = 0;
-  }
+  g_clear_handle_id (&action_bar_start->navigation_buttons_menu_timeout, g_source_remove);
 
   G_OBJECT_CLASS (ephy_action_bar_start_parent_class)->dispose (object);
 }
 
+static void
+update_new_tab_button_visibility (EphyActionBarStart *action_bar_start)
+{
+  EphyEmbedShell *embed_shell;
+
+  embed_shell = ephy_embed_shell_get_default ();
+
+  gtk_widget_set_visible (action_bar_start->new_tab_button,
+                          (ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_APPLICATION) &&
+                          !is_desktop_pantheon () &&
+                          !action_bar_start->is_fullscreen);
+}
+
 static void
 ephy_action_bar_start_constructed (GObject *object)
 {
   EphyActionBarStart *action_bar_start = EPHY_ACTION_BAR_START (object);
   EphyEmbedShell *embed_shell;
 
+  G_OBJECT_CLASS (ephy_action_bar_start_parent_class)->constructed (object);
+
   gtk_widget_init_template (GTK_WIDGET (action_bar_start));
 
   /* Back */
@@ -498,10 +543,11 @@ ephy_action_bar_start_constructed (GObject *object)
   embed_shell = ephy_embed_shell_get_default ();
   if (ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_APPLICATION) {
     homepage_url_changed (EPHY_SETTINGS_MAIN, EPHY_PREFS_HOMEPAGE_URL, action_bar_start->homepage_button);
-    g_signal_connect (EPHY_SETTINGS_MAIN,
-                      "changed::" EPHY_PREFS_HOMEPAGE_URL,
-                      G_CALLBACK (homepage_url_changed),
-                      action_bar_start->homepage_button);
+    g_signal_connect_object (EPHY_SETTINGS_MAIN,
+                             "changed::" EPHY_PREFS_HOMEPAGE_URL,
+                             G_CALLBACK (homepage_url_changed),
+                             action_bar_start->homepage_button,
+                             0);
   } else {
     gtk_widget_set_visible (action_bar_start->homepage_button, FALSE);
   }
@@ -509,11 +555,28 @@ ephy_action_bar_start_constructed (GObject *object)
                     G_CALLBACK (homepage_button_release_event_cb), action_bar_start);
 
   /* New Tab Button */
-  if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
-    gtk_widget_set_visible (action_bar_start->new_tab_button, FALSE);
+  update_new_tab_button_visibility (action_bar_start);
 
   g_signal_connect (action_bar_start->new_tab_button, "button-release-event",
                     G_CALLBACK (new_tab_button_release_event_cb), action_bar_start);
+
+  if (is_desktop_pantheon ()) {
+    gtk_button_set_image (GTK_BUTTON (action_bar_start->navigation_back),
+                          gtk_image_new_from_icon_name ("go-previous-symbolic",
+                                                        get_icon_size ()));
+    gtk_button_set_image (GTK_BUTTON (action_bar_start->navigation_forward),
+                          gtk_image_new_from_icon_name ("go-next-symbolic",
+                                                        get_icon_size ()));
+    gtk_button_set_image (GTK_BUTTON (action_bar_start->homepage_button),
+                          gtk_image_new_from_icon_name ("go-home-symbolic",
+                                                        get_icon_size ()));
+  }
+
+  if (ephy_profile_dir_is_web_application ()) {
+    GtkWidget *navigation_box = ephy_action_bar_start_get_navigation_box (action_bar_start);
+
+    g_settings_bind (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_MOBILE_CAPABLE, navigation_box, "visible", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_INVERT_BOOLEAN);
+  }
 }
 
 static void
@@ -549,11 +612,15 @@ ephy_action_bar_start_class_init (EphyActionBarStartClass *klass)
   gtk_widget_class_bind_template_child (widget_class,
                                         EphyActionBarStart,
                                         new_tab_button);
+  gtk_widget_class_bind_template_child (widget_class,
+                                        EphyActionBarStart,
+                                        placeholder);
 }
 
 static void
 ephy_action_bar_start_init (EphyActionBarStart *action_bar_start)
 {
+  action_bar_start->is_fullscreen = FALSE;
 }
 
 EphyActionBarStart *
@@ -574,17 +641,56 @@ ephy_action_bar_start_change_combined_stop_reload_state (EphyActionBarStart *act
                                                          gboolean            loading)
 {
   if (loading) {
-        gtk_image_set_from_icon_name (GTK_IMAGE (action_bar_start->combined_stop_reload_image),
+    gtk_image_set_from_icon_name (GTK_IMAGE (action_bar_start->combined_stop_reload_image),
                                   "process-stop-symbolic",
-                                  GTK_ICON_SIZE_BUTTON);
+                                  get_icon_size ());
     /* Translators: tooltip for the stop button */
     gtk_widget_set_tooltip_text (action_bar_start->combined_stop_reload_button,
                                  _("Stop loading the current page"));
   } else {
-        gtk_image_set_from_icon_name (GTK_IMAGE (action_bar_start->combined_stop_reload_image),
-                                      "view-refresh-symbolic",
-                                      GTK_ICON_SIZE_BUTTON);
+    gtk_image_set_from_icon_name (GTK_IMAGE (action_bar_start->combined_stop_reload_image),
+                                  "view-refresh-symbolic",
+                                  get_icon_size ());
     gtk_widget_set_tooltip_text (action_bar_start->combined_stop_reload_button,
                                  _(REFRESH_BUTTON_TOOLTIP));
   }
 }
+
+void
+ephy_action_bar_set_is_fullscreen (EphyActionBarStart *action_bar_start,
+                                   gboolean            fullscreen)
+{
+  if (action_bar_start->is_fullscreen == fullscreen)
+    return;
+
+  action_bar_start->is_fullscreen = fullscreen;
+  update_new_tab_button_visibility (action_bar_start);
+}
+
+GtkWidget *
+ephy_action_bar_start_get_placeholder (EphyActionBarStart *action_bar_start)
+{
+  return action_bar_start->placeholder;
+}
+
+void
+ephy_action_bar_start_set_adaptive_mode (EphyActionBarStart *action_bar,
+                                         EphyAdaptiveMode    adaptive_mode)
+{
+  GValue val = G_VALUE_INIT;
+
+  g_value_init (&val, G_TYPE_INT);
+
+  gtk_widget_set_visible (action_bar->new_tab_button, adaptive_mode == EPHY_ADAPTIVE_MODE_NORMAL);
+  gtk_widget_set_visible (action_bar->combined_stop_reload_button, adaptive_mode == EPHY_ADAPTIVE_MODE_NORMAL);
+
+  if (adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW)
+    g_value_set_int (&val, 42);
+  else
+    g_value_set_int (&val, -1);
+
+  g_object_set_property (G_OBJECT (action_bar->navigation_back), "width-request", &val);
+  g_object_set_property (G_OBJECT (action_bar->navigation_forward), "width-request", &val);
+
+  g_value_unset (&val);
+}
