@@ -23,6 +23,8 @@
 
 #include <gtk/gtk.h>
 
+#include "ephy-adaptive-mode.h"
+
 G_BEGIN_DECLS
 
 #define EPHY_TYPE_ACTION_BAR_START (ephy_action_bar_start_get_type ())
@@ -37,4 +39,7 @@ void                ephy_action_bar_set_is_fullscreen                       (Eph
                                                                              gboolean            fullscreen);
 GtkWidget          *ephy_action_bar_start_get_placeholder                   (EphyActionBarStart *action_bar_start);
 
+void                ephy_action_bar_start_set_adaptive_mode                 (EphyActionBarStart *action_bar,
+                                                                             EphyAdaptiveMode    adaptive_mode);
+
 G_END_DECLS
