@@ -26,16 +26,17 @@
 #include "ephy-add-bookmark-popover.h"
 #include "ephy-desktop-utils.h"
 #include "ephy-embed-utils.h"
+#include "ephy-file-helpers.h"
 #include "ephy-flatpak-utils.h"
 #include "ephy-location-entry.h"
 #include "ephy-notebook.h"
+#include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-title-box.h"
 #include "ephy-title-widget.h"
 #include "ephy-type-builtins.h"
 
 #include <glib/gi18n.h>
-#define HANDY_USE_UNSTABLE_API
 #include <handy.h>
 
 enum {
@@ -46,6 +47,9 @@ enum {
 
 static GParamSpec *object_properties[N_PROPERTIES] = { NULL, };
 
+/* Translators: tooltip for the refresh button */
+static const char *REFRESH_BUTTON_TOOLTIP = N_("Reload the current page");
+
 struct _EphyHeaderBar {
   GtkHeaderBar parent_instance;
 
@@ -58,6 +62,8 @@ struct _EphyHeaderBar {
   GtkWidget *page_menu_button;
   GtkWidget *zoom_level_button;
   GtkWidget *restore_button;
+  GtkWidget *combined_stop_reload_button;
+  GtkWidget *combined_stop_reload_image;
 };
 
 G_DEFINE_TYPE (EphyHeaderBar, ephy_header_bar, GTK_TYPE_HEADER_BAR)
@@ -158,42 +164,13 @@ update_revealer_visibility (GtkRevealer *revealer)
                           gtk_revealer_get_child_revealed (revealer));
 }
 
-static void
-handle_primary_tab_key (GtkWidget *widget,
-                        gpointer   user_data)
-{
-  EphyHeaderBar *header_bar = EPHY_HEADER_BAR (user_data);
-  GtkWidget *entry = ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (header_bar->title_widget));
-  GtkWidget *nb;
-
-  nb = ephy_window_get_notebook (header_bar->window);
-  g_assert (nb != NULL);
-
-  ephy_notebook_next_page (EPHY_NOTEBOOK (nb));
-  gtk_widget_grab_focus (entry);
-}
-
-static void
-handle_primary_iso_tab_key (GtkWidget *widget,
-                            gpointer   user_data)
-{
-  EphyHeaderBar *header_bar = EPHY_HEADER_BAR (user_data);
-  GtkWidget *entry = ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (header_bar->title_widget));
-  GtkWidget *nb;
-
-  nb = ephy_window_get_notebook (header_bar->window);
-  g_assert (nb != NULL);
-
-  ephy_notebook_prev_page (EPHY_NOTEBOOK (nb));
-  gtk_widget_grab_focus (entry);
-}
-
 static void
 ephy_header_bar_constructed (GObject *object)
 {
   EphyHeaderBar *header_bar = EPHY_HEADER_BAR (object);
   GtkWidget *button;
   GtkWidget *page_menu_popover;
+  GtkWidget *event_box;
   GtkBuilder *builder;
   EphyEmbedShell *embed_shell;
   GtkSizeGroup *downloads_size_group;
@@ -227,48 +204,33 @@ ephy_header_bar_constructed (GObject *object)
   if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_title_box_new ());
   else {
-    DzlShortcutController *controller;
-    GtkWidget *entry;
-
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_location_entry_new ());
-    entry = ephy_location_entry_get_entry (EPHY_LOCATION_ENTRY (header_bar->title_widget));
-    controller = dzl_shortcut_controller_find (entry);
-
-    dzl_shortcut_controller_add_command_callback (controller,
-                                                  "org.gnome.Epiphany.next-tab-pages",
-                                                  "<Primary>Tab",
-                                                  DZL_SHORTCUT_PHASE_DISPATCH,
-                                                  handle_primary_tab_key,
-                                                  header_bar,
-                                                  NULL);
-
-    dzl_shortcut_controller_add_command_callback (controller,
-                                                  "org.gnome.Epiphany.prev-tab-pages",
-                                                  "<Primary>ISO_Left_Tab",
-                                                  DZL_SHORTCUT_PHASE_DISPATCH,
-                                                  handle_primary_iso_tab_key,
-                                                  header_bar,
-                                                  NULL);
   }
 
+  event_box = gtk_event_box_new ();
+  gtk_widget_add_events (event_box, GDK_ALL_EVENTS_MASK);
+  gtk_widget_show (event_box);
+  gtk_header_bar_set_custom_title (GTK_HEADER_BAR (header_bar), event_box);
+  gtk_widget_set_name (event_box, "title-box-container");
+
   if (is_desktop_pantheon ()) {
     /* Use a full-width entry on Pantheon */
     gtk_widget_set_hexpand (GTK_WIDGET (header_bar->title_widget), TRUE);
     gtk_widget_set_margin_start (GTK_WIDGET (header_bar->title_widget), 6);
     gtk_widget_set_margin_end (GTK_WIDGET (header_bar->title_widget), 6);
 
-    gtk_header_bar_set_custom_title (GTK_HEADER_BAR (header_bar), GTK_WIDGET (header_bar->title_widget));
+    gtk_container_add (GTK_CONTAINER (event_box), GTK_WIDGET (header_bar->title_widget));
   } else {
-    HdyColumn *column;
+    GtkWidget *clamp;
 
-    column = hdy_column_new ();
-    gtk_widget_set_hexpand (GTK_WIDGET (column), TRUE);
-    gtk_widget_show (GTK_WIDGET (column));
-    hdy_column_set_maximum_width (column, 860);
-    hdy_column_set_linear_growth_width (column, 560);
-    gtk_container_add (GTK_CONTAINER (column), GTK_WIDGET (header_bar->title_widget));
+    clamp = hdy_clamp_new ();
+    gtk_widget_set_hexpand (GTK_WIDGET (clamp), TRUE);
+    gtk_widget_show (clamp);
+    hdy_clamp_set_maximum_size (HDY_CLAMP (clamp), 860);
+    hdy_clamp_set_tightening_threshold (HDY_CLAMP (clamp), 560);
+    gtk_container_add (GTK_CONTAINER (clamp), GTK_WIDGET (header_bar->title_widget));
 
-    gtk_header_bar_set_custom_title (GTK_HEADER_BAR (header_bar), GTK_WIDGET (column));
+    gtk_container_add (GTK_CONTAINER (event_box), clamp);
   }
 
   gtk_widget_show (GTK_WIDGET (header_bar->title_widget));
@@ -309,9 +271,6 @@ ephy_header_bar_constructed (GObject *object)
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "new-window-button")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "new-incognito-window-button")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "reopen-closed-tab-button")));
-    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "bookmarks-separator")));
-    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "import-bookmarks-button")));
-    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "export-bookmarks-button")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "save-as-application-separator")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "save-as-application-button")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "application-manager-button")));
@@ -319,15 +278,26 @@ ephy_header_bar_constructed (GObject *object)
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "override-text-encoding-button")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "keyboard-shortcuts-button")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "help-button")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "import-export-separator")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "import-export-menu")));
   } else if (ephy_is_running_inside_flatpak ()) {
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "run-in-background-separator")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "run-in-background-button")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "save-as-application-separator")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "save-as-application-button")));
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "application-manager-button")));
 
     if (is_desktop_pantheon ())
       gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "help-button")));
+  } else {
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "run-in-background-separator")));
+    gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "run-in-background-button")));
   }
 
+  header_bar->combined_stop_reload_button = GTK_WIDGET (gtk_builder_get_object (builder, "combined_stop_reload_button"));
+  header_bar->combined_stop_reload_image = GTK_WIDGET (gtk_builder_get_object (builder, "combined_stop_reload_image"));
+  gtk_widget_set_tooltip_text (header_bar->combined_stop_reload_button, _(REFRESH_BUTTON_TOOLTIP));
+
   if (is_desktop_pantheon ()) {
     gtk_widget_destroy (GTK_WIDGET (gtk_builder_get_object (builder, "about-button")));
 
@@ -362,6 +332,12 @@ ephy_header_bar_constructed (GObject *object)
   gtk_size_group_add_widget (downloads_size_group,
                              ephy_action_bar_end_get_downloads_revealer (header_bar->action_bar_end));
   g_object_unref (downloads_size_group);
+
+  if (ephy_profile_dir_is_web_application ()) {
+    GtkWidget *navigation_box = ephy_action_bar_start_get_navigation_box (header_bar->action_bar_start);
+
+    g_settings_bind (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_MOBILE_CAPABLE, navigation_box, "visible", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_INVERT_BOOLEAN);
+  }
 }
 
 static void
@@ -445,11 +421,13 @@ ephy_header_bar_set_adaptive_mode (EphyHeaderBar    *header_bar,
     case EPHY_ADAPTIVE_MODE_NORMAL:
       gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->start_revealer), TRUE);
       gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->end_revealer), TRUE);
+      gtk_widget_set_visible (header_bar->combined_stop_reload_button, FALSE);
 
       break;
     case EPHY_ADAPTIVE_MODE_NARROW:
       gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->start_revealer), FALSE);
       gtk_revealer_set_reveal_child (GTK_REVEALER (header_bar->end_revealer), FALSE);
+      gtk_widget_set_visible (header_bar->combined_stop_reload_button, TRUE);
 
       break;
   }
@@ -457,3 +435,23 @@ ephy_header_bar_set_adaptive_mode (EphyHeaderBar    *header_bar,
   if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) != EPHY_EMBED_SHELL_MODE_APPLICATION)
     ephy_location_entry_set_mobile_popdown (EPHY_LOCATION_ENTRY (header_bar->title_widget), adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW);
 }
+
+void
+ephy_header_bar_start_change_combined_stop_reload_state (EphyHeaderBar *header_bar,
+                                                         gboolean       loading)
+{
+  if (loading) {
+    gtk_image_set_from_icon_name (GTK_IMAGE (header_bar->combined_stop_reload_image),
+                                  "process-stop-symbolic",
+                                  get_icon_size ());
+    /* Translators: tooltip for the stop button */
+    gtk_widget_set_tooltip_text (header_bar->combined_stop_reload_button,
+                                 _("Stop loading the current page"));
+  } else {
+    gtk_image_set_from_icon_name (GTK_IMAGE (header_bar->combined_stop_reload_image),
+                                  "view-refresh-symbolic",
+                                  get_icon_size ());
+    gtk_widget_set_tooltip_text (header_bar->combined_stop_reload_button,
+                                 _(REFRESH_BUTTON_TOOLTIP));
+  }
+}
