@@ -45,5 +45,7 @@ EphyActionBarStart *ephy_header_bar_get_action_bar_start           (EphyHeaderBa
 EphyActionBarEnd   *ephy_header_bar_get_action_bar_end             (EphyHeaderBar *header_bar);
 void                ephy_header_bar_set_adaptive_mode              (EphyHeaderBar    *header_bar,
                                                                     EphyAdaptiveMode  adaptive_mode);
+void                ephy_header_bar_start_change_combined_stop_reload_state (EphyHeaderBar *header_bar,
+                                                                             gboolean       loading);
 
 G_END_DECLS
