@@ -24,6 +24,7 @@
 #include "ephy-notebook.h"
 
 #include "ephy-debug.h"
+#include "ephy-desktop-utils.h"
 #include "ephy-dnd.h"
 #include "ephy-embed-utils.h"
 #include "ephy-embed.h"
@@ -53,7 +54,6 @@ struct _EphyNotebook {
   EphyAdaptiveMode adaptive_mode;
 
   GList *focused_pages;
-  guint tabs_vis_notifier_id;
 
   GMenu *tab_menu;
 
@@ -370,8 +370,11 @@ update_tabs_visibility (EphyNotebook *nb,
   if (before_inserting)
     num++;
 
-  policy = g_settings_get_enum (EPHY_SETTINGS_UI,
-                                EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY);
+  if (is_desktop_pantheon ())
+    policy = EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY_ALWAYS;
+  else
+    policy = g_settings_get_enum (EPHY_SETTINGS_UI,
+                                  EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY);
 
   if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION &&
       nb->adaptive_mode != EPHY_ADAPTIVE_MODE_NARROW &&
@@ -383,6 +386,15 @@ update_tabs_visibility (EphyNotebook *nb,
   gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb), nb->tabs_allowed && show_tabs);
 }
 
+static gboolean
+expand_tabs_bar (void)
+{
+  if (is_desktop_pantheon ())
+    return FALSE;
+  return g_settings_get_boolean (EPHY_SETTINGS_UI,
+                                 EPHY_PREFS_UI_EXPAND_TABS_BAR);
+}
+
 static void
 expand_tabs_changed_cb (GSettings    *settings,
                         char         *key,
@@ -392,8 +404,7 @@ expand_tabs_changed_cb (GSettings    *settings,
   GList *l;
   gboolean expand;
 
-  expand = g_settings_get_boolean (EPHY_SETTINGS_UI,
-                                   EPHY_PREFS_UI_EXPAND_TABS_BAR),
+  expand = expand_tabs_bar ();
 
   tabs = gtk_container_get_children (GTK_CONTAINER (nb));
 
@@ -492,21 +503,53 @@ get_last_pinned_tab_pos (EphyNotebook *notebook)
   return found;
 }
 
+static int
+get_first_unpinned_tab_pos (EphyNotebook *notebook)
+{
+  int pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+
+  for (int i = 0; i < pages; i++) {
+    GtkWidget *child = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i);
+
+    if (!ephy_notebook_tab_is_pinned (notebook, EPHY_EMBED (child)))
+      return i;
+  }
+
+  return pages;
+}
+
+static void
+ephy_notebook_ensure_pinned_tab_position (GtkNotebook *notebook,
+                                          GtkWidget   *child,
+                                          guint        page_num)
+{
+  int last_pinned_tab_pos = get_last_pinned_tab_pos (EPHY_NOTEBOOK (notebook));
+  int first_unpinned_tab_pos = get_first_unpinned_tab_pos (EPHY_NOTEBOOK (notebook));
+  gboolean is_current_tab_pinned = ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), EPHY_EMBED (child));
+
+  /* Ensure that pinned tabs will always stay at the beginning of tab bar */
+  if (!is_current_tab_pinned) {
+    if (last_pinned_tab_pos != -1 && page_num <= (guint)last_pinned_tab_pos) {
+      gtk_notebook_reorder_child (notebook, child, last_pinned_tab_pos);
+    }
+  } else {
+    if (first_unpinned_tab_pos != -1) {
+      if (page_num > (guint)first_unpinned_tab_pos) {
+        gtk_notebook_reorder_child (notebook, child, first_unpinned_tab_pos);
+      }
+    } else {
+      gtk_notebook_reorder_child (notebook, child, 0);
+    }
+  }
+}
+
 static void
 page_reordered_cb (GtkNotebook *notebook,
                    GtkWidget   *child,
                    guint        page_num,
                    gpointer     user_data)
 {
-  int last_pinned_tab_pos = get_last_pinned_tab_pos (EPHY_NOTEBOOK (notebook));
-  gboolean is_current_tab_pinned = ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), EPHY_EMBED (child));
-
-  /* Ensure that pinned tabs will always stay at the beginning of tab bar */
-  if (last_pinned_tab_pos != -1 && page_num <= (guint)last_pinned_tab_pos) {
-    gint new_pos = is_current_tab_pinned ? last_pinned_tab_pos : last_pinned_tab_pos + 1;
-
-    gtk_notebook_reorder_child (notebook, child, new_pos);
-  }
+  ephy_notebook_ensure_pinned_tab_position (notebook, child, page_num);
 }
 
 static void
@@ -557,21 +600,26 @@ static void
 ephy_notebook_constructed (GObject *object)
 {
   EphyNotebook *notebook = EPHY_NOTEBOOK (object);
-  GtkWidget *hbox;
   GtkWidget *button;
   EphyPagesPopover *popover;
 
   G_OBJECT_CLASS (ephy_notebook_parent_class)->constructed (object);
 
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_notebook_set_action_widget (GTK_NOTEBOOK (notebook), hbox, GTK_PACK_END);
-  gtk_widget_show (hbox);
+  if (is_desktop_pantheon ()) {
+    button = gtk_button_new_from_icon_name ("tab-new-symbolic", GTK_ICON_SIZE_MENU);
+    /* Translators: tooltip for the new tab button */
+    gtk_widget_set_tooltip_text (button, _("Open a new tab"));
+    gtk_actionable_set_action_name (GTK_ACTIONABLE (button), "win.new-tab");
+    gtk_style_context_add_class (gtk_widget_get_style_context (button), "flat");
+    gtk_notebook_set_action_widget (GTK_NOTEBOOK (notebook), button, GTK_PACK_START);
+    gtk_widget_show (button);
+  }
 
   button = gtk_menu_button_new ();
   gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
   /* Translators: tooltip for the tab switcher menu button */
   gtk_widget_set_tooltip_text (button, _("View open tabs"));
-  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  gtk_notebook_set_action_widget (GTK_NOTEBOOK (notebook), button, GTK_PACK_END);
   gtk_widget_show (button);
 
   notebook->tab_menu = g_menu_new ();
@@ -713,6 +761,16 @@ close_button_clicked_cb (GtkWidget *widget,
   g_signal_emit (notebook, signals[TAB_CLOSE_REQUEST], 0, tab);
 }
 
+static void
+audio_clicked_cb (GtkWidget *widget,
+                  EphyEmbed *embed)
+{
+  EphyWebView *view = ephy_embed_get_web_view (embed);
+  gboolean muted = webkit_web_view_get_is_muted (WEBKIT_WEB_VIEW (view));
+
+  webkit_web_view_set_is_muted (WEBKIT_WEB_VIEW (view), !muted);
+}
+
 static GtkWidget *
 build_tab_label (EphyNotebook *nb,
                  EphyEmbed    *embed)
@@ -722,6 +780,7 @@ build_tab_label (EphyNotebook *nb,
 
   tab_label = ephy_tab_label_new ();
   g_signal_connect (tab_label, "close-clicked", G_CALLBACK (close_button_clicked_cb), embed);
+  g_signal_connect (tab_label, "audio-clicked", G_CALLBACK (audio_clicked_cb), embed);
 
   /* Set up drag-and-drop target */
   g_signal_connect (tab_label, "drag-data-received",
@@ -737,11 +796,12 @@ build_tab_label (EphyNotebook *nb,
   g_signal_connect_object (embed, "notify::title",
                            G_CALLBACK (title_changed_cb), nb, 0);
 
-  g_object_bind_property (view, "title", tab_label, "label-text", G_BINDING_DEFAULT);
-  g_object_bind_property (view, "display-address", tab_label, "label-uri", G_BINDING_DEFAULT);
-  g_object_bind_property (view, "icon", tab_label, "icon-buf", G_BINDING_DEFAULT);
-  g_object_bind_property (view, "is-loading", tab_label, "spinning", G_BINDING_DEFAULT);
-  g_object_bind_property (view, "is-playing-audio", tab_label, "audio", G_BINDING_DEFAULT);
+  g_object_bind_property (embed, "title", tab_label, "label-text", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+  g_object_bind_property (view, "display-address", tab_label, "label-uri", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+  g_object_bind_property (view, "icon", tab_label, "icon-buf", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+  g_object_bind_property (view, "is-loading", tab_label, "spinning", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+  g_object_bind_property (view, "is-playing-audio", tab_label, "audio", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+  g_object_bind_property (view, "is-muted", tab_label, "audio-muted", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
 
   return tab_label;
 }
@@ -788,13 +848,89 @@ ephy_notebook_insert_page (GtkNotebook *gnotebook,
   gtk_notebook_set_tab_detachable (gnotebook, tab_widget, TRUE);
   gtk_container_child_set (GTK_CONTAINER (gnotebook),
                            GTK_WIDGET (tab_widget),
-                           "tab-expand", g_settings_get_boolean (EPHY_SETTINGS_UI,
-                                                                 EPHY_PREFS_UI_EXPAND_TABS_BAR),
+                           "tab-expand", expand_tabs_bar (),
                            NULL);
 
   return position;
 }
 
+typedef struct {
+  EphyNotebook *notebook;
+  GtkTextDirection direction;
+  int old_pos;
+  int new_pos;
+  guint flash_id;
+} PresentTabData;
+
+static PresentTabData *
+present_tab_data_new (EphyNotebook *notebook,
+                      gint          old_pos,
+                      gint          new_pos)
+{
+  PresentTabData *pt_data = g_new0 (PresentTabData, 1);
+  pt_data->notebook = notebook;
+  pt_data->old_pos = old_pos;
+  pt_data->new_pos = new_pos;
+  pt_data->direction = gtk_widget_get_default_direction ();
+  g_object_add_weak_pointer (G_OBJECT (notebook), (gpointer *)&pt_data->notebook);
+  return pt_data;
+}
+
+static void
+present_tab_data_free (PresentTabData *pt_data)
+{
+  g_clear_weak_pointer (&pt_data->notebook);
+  g_clear_handle_id (&pt_data->flash_id, g_source_remove);
+  g_clear_pointer (&pt_data, g_free);
+}
+
+static gboolean
+remove_arrow_flash (gpointer user_data)
+{
+  PresentTabData *pt_data = user_data;
+
+  if (pt_data->direction == GTK_TEXT_DIR_LTR)
+    gtk_style_context_remove_class (gtk_widget_get_style_context (GTK_WIDGET (pt_data->notebook)), "tab-arrow-up-attention");
+  else
+    gtk_style_context_remove_class (gtk_widget_get_style_context (GTK_WIDGET (pt_data->notebook)), "tab-arrow-down-attention");
+
+  pt_data->flash_id = 0;
+  present_tab_data_free (pt_data);
+
+  return G_SOURCE_REMOVE;
+}
+
+static gboolean
+present_new_tab (gpointer user_data)
+{
+  PresentTabData *pt_data = user_data;
+  GtkWidget *page;
+  GtkWidget *label;
+
+  if (!pt_data->notebook) {
+    pt_data->flash_id = 0;
+    present_tab_data_free (pt_data);
+    return G_SOURCE_REMOVE;
+  }
+
+  page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (pt_data->notebook), pt_data->new_pos);
+  label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (pt_data->notebook), page);
+
+  if (!gtk_widget_get_mapped (GTK_WIDGET (label))) {
+    if (pt_data->direction == GTK_TEXT_DIR_LTR)
+      gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (pt_data->notebook)), "tab-arrow-up-attention");
+    else
+      gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (pt_data->notebook)), "tab-arrow-down-attention");
+
+    pt_data->flash_id = g_timeout_add (500, remove_arrow_flash, pt_data);
+  } else {
+    pt_data->flash_id = 0;
+    present_tab_data_free (pt_data);
+  }
+
+  return G_SOURCE_REMOVE;
+}
+
 int
 ephy_notebook_add_tab (EphyNotebook *notebook,
                        EphyEmbed    *embed,
@@ -812,14 +948,16 @@ ephy_notebook_add_tab (EphyNotebook *notebook,
 
   gtk_container_child_set (GTK_CONTAINER (notebook),
                            GTK_WIDGET (embed),
-                           "tab-expand", g_settings_get_boolean (EPHY_SETTINGS_UI,
-                                                                 EPHY_PREFS_UI_EXPAND_TABS_BAR),
+                           "tab-expand", expand_tabs_bar (),
                            NULL);
 
   if (jump_to) {
     gtk_notebook_set_current_page (gnotebook, position);
     g_object_set_data (G_OBJECT (embed), "jump_to",
                        GINT_TO_POINTER (jump_to));
+  } else if (ephy_shell_startup_finished (ephy_shell_get_default ())) {
+    PresentTabData *pt_data = present_tab_data_new (notebook, gtk_notebook_get_current_page (gnotebook), position);
+    pt_data->flash_id = g_idle_add (present_new_tab, pt_data);
   }
 
   return position;
@@ -886,6 +1024,8 @@ ephy_notebook_page_added (GtkNotebook *notebook,
   if (GTK_NOTEBOOK_CLASS (ephy_notebook_parent_class)->page_added != NULL)
     GTK_NOTEBOOK_CLASS (ephy_notebook_parent_class)->page_added (notebook, child, page_num);
 
+  ephy_notebook_ensure_pinned_tab_position (notebook, child, page_num);
+
   ephy_notebook_rebuild_tab_menu (EPHY_NOTEBOOK (notebook));
 }
 
@@ -997,17 +1137,21 @@ ephy_notebook_tab_set_pinned (EphyNotebook *notebook,
                               gboolean      is_pinned)
 {
   GtkWidget *tab_label;
+  int last_pinned_tab;
   gboolean expanded;
 
-  gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (notebook), embed, !is_pinned);
+  gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (notebook), embed, !is_pinned);
   tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (notebook), embed);
+
+  /* We have to compute the last pinned tab *before* we pin/unpin this tab. */
+  last_pinned_tab = get_last_pinned_tab_pos (notebook);
   ephy_tab_label_set_pinned (tab_label, is_pinned);
 
   if (is_pinned) {
-    gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), embed, 0);
+    gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), embed, last_pinned_tab != -1 ? last_pinned_tab + 1 : 0);
     expanded = FALSE;
   } else {
-    expanded = g_settings_get_boolean (EPHY_SETTINGS_UI, EPHY_PREFS_UI_EXPAND_TABS_BAR);
+    expanded = expand_tabs_bar ();
     gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), embed, -1);
   }
 
