@@ -21,6 +21,7 @@
 
 #include "config.h"
 
+#include "ephy-desktop-utils.h"
 #include "ephy-embed-utils.h"
 #include "ephy-page-row.h"
 #include "ephy-web-view.h"
@@ -140,11 +141,19 @@ sync_favicon (EphyWebView *view,
   if (ephy_web_view_get_icon (view))
     pixbuf = gdk_pixbuf_copy (ephy_web_view_get_icon (view));
 
-  if (pixbuf)
-    gtk_image_set_from_pixbuf (self->icon, pixbuf);
-  else
-    gtk_image_set_from_icon_name (self->icon, "ephy-missing-favicon-symbolic",
-                                  GTK_ICON_SIZE_MENU);
+  if (pixbuf) {
+    cairo_surface_t *surface;
+
+    surface = gdk_cairo_surface_create_from_pixbuf (pixbuf,
+                                                    0,
+                                                    gtk_widget_get_window (GTK_WIDGET (view)));
+    gtk_image_set_from_surface (GTK_IMAGE (self->icon), surface);
+    cairo_surface_destroy (surface);
+  } else {
+    const char *favicon_name = ephy_get_fallback_favicon_name (ephy_web_view_get_display_address (view), EPHY_FAVICON_TYPE_SHOW_MISSING_PLACEHOLDER);
+
+    gtk_image_set_from_icon_name (GTK_IMAGE (self->icon), favicon_name, GTK_ICON_SIZE_MENU);
+  }
 }
 
 EphyPageRow *
