@@ -64,10 +64,10 @@ struct _EphySession {
   GCancellable *save_cancellable;
   guint closing : 1;
   guint dont_save : 1;
+  guint loaded_page : 1;
 };
 
 #define SESSION_STATE           "type:session_state"
-#define MAX_CLOSED_TABS         10
 
 enum {
   PROP_0,
@@ -118,12 +118,17 @@ session_delete (EphySession *session)
 }
 
 static void
-load_changed_cb (WebKitWebView  *view,
-                 WebKitLoadEvent load_event,
-                 EphySession    *session)
+load_changed_cb (WebKitWebView   *view,
+                 WebKitLoadEvent  load_event,
+                 EphySession     *session)
 {
-  if (!ephy_web_view_load_failed (EPHY_WEB_VIEW (view)))
-    ephy_session_save (session);
+  if (ephy_web_view_load_failed (EPHY_WEB_VIEW (view)))
+    return;
+
+  if (load_event == WEBKIT_LOAD_FINISHED)
+    session->loaded_page = TRUE;
+
+  ephy_session_save (session);
 }
 
 static void
@@ -179,7 +184,8 @@ closed_tab_get_notebook (ClosedTab *tab)
 }
 
 static int
-compare_func (ClosedTab *iter, EphyNotebook *notebook)
+compare_func (ClosedTab    *iter,
+              EphyNotebook *notebook)
 {
   return GTK_NOTEBOOK (closed_tab_get_notebook (iter)) - GTK_NOTEBOOK (notebook);
 }
@@ -299,10 +305,6 @@ ephy_session_tab_closed (EphySession  *session,
     return;
   }
 
-  if (g_queue_get_length (session->closed_tabs) == MAX_CLOSED_TABS) {
-    closed_tab_free (g_queue_pop_tail (session->closed_tabs));
-  }
-
   tab = closed_tab_new (view, position,
                         ephy_session_ref_or_create_notebook_tracker (session, notebook));
   g_queue_push_head (session->closed_tabs, tab);
@@ -509,13 +511,10 @@ ephy_session_close (EphySession *session)
 
   LOG ("ephy_session_close");
 
-  if (session->save_source_id) {
-    /* There's a save pending, cancel it and save the session now since
-     * after closing the session the saving is no longer allowed.
-     */
-    g_source_remove (session->save_source_id);
-    session->save_source_id = 0;
-  }
+  /* If there's a save pending, cancel it and save the session now since
+   * after closing the session the saving is no longer allowed.
+   */
+  g_clear_handle_id (&session->save_source_id, g_source_remove);
 
   if (session->closing)
     return;
@@ -532,12 +531,24 @@ ephy_session_close (EphySession *session)
   session->dont_save = TRUE;
 }
 
+typedef struct {
+  GdkRectangle geometry;
+  gboolean is_maximized;
+  gboolean is_fullscreen;
+  char *role;
+
+  GList *tabs;
+  gint active_tab;
+} SessionWindow;
+
 static void
-get_window_geometry (GtkWindow    *window,
-                     GdkRectangle *rectangle)
+get_window_geometry (EphyWindow    *window,
+                     SessionWindow *session_window)
 {
-  gtk_window_get_size (window, &rectangle->width, &rectangle->height);
-  gtk_window_get_position (window, &rectangle->x, &rectangle->y);
+  session_window->is_maximized = ephy_window_is_maximized (window);
+  session_window->is_fullscreen = ephy_window_is_fullscreen (window);
+
+  ephy_window_get_geometry (window, &session_window->geometry);
 }
 
 typedef struct {
@@ -545,12 +556,14 @@ typedef struct {
   char *title;
   gboolean loading;
   gboolean crashed;
+  gboolean pinned;
   WebKitWebViewSessionState *state;
 } SessionTab;
 
 static SessionTab *
 session_tab_new (EphyEmbed   *embed,
-                 EphySession *session)
+                 EphySession *session,
+                 GtkNotebook *notebook)
 {
   SessionTab *session_tab;
   const char *address;
@@ -579,6 +592,7 @@ session_tab_new (EphyEmbed   *embed,
   session_tab->crashed = (error_page == EPHY_WEB_VIEW_ERROR_PAGE_CRASH ||
                           error_page == EPHY_WEB_VIEW_ERROR_PROCESS_CRASH);
   session_tab->state = webkit_web_view_get_session_state (WEBKIT_WEB_VIEW (web_view));
+  session_tab->pinned = ephy_notebook_tab_is_pinned (EPHY_NOTEBOOK (notebook), embed);
 
   return session_tab;
 }
@@ -593,14 +607,6 @@ session_tab_free (SessionTab *tab)
   g_free (tab);
 }
 
-typedef struct {
-  GdkRectangle geometry;
-  char *role;
-
-  GList *tabs;
-  gint active_tab;
-} SessionWindow;
-
 static SessionWindow *
 session_window_new (EphyWindow  *window,
                     EphySession *session)
@@ -618,19 +624,19 @@ session_window_new (EphyWindow  *window,
   }
 
   session_window = g_new0 (SessionWindow, 1);
-  get_window_geometry (GTK_WINDOW (window), &session_window->geometry);
+  get_window_geometry (window, session_window);
   session_window->role = g_strdup (gtk_window_get_role (GTK_WINDOW (window)));
+  notebook = GTK_NOTEBOOK (ephy_window_get_notebook (window));
 
   for (l = tabs; l != NULL; l = l->next) {
     SessionTab *tab;
 
-    tab = session_tab_new (EPHY_EMBED (l->data), session);
+    tab = session_tab_new (EPHY_EMBED (l->data), session, notebook);
     session_window->tabs = g_list_prepend (session_window->tabs, tab);
   }
   g_list_free (tabs);
   session_window->tabs = g_list_reverse (session_window->tabs);
 
-  notebook = GTK_NOTEBOOK (ephy_window_get_notebook (window));
   session_window->active_tab = gtk_notebook_get_current_page (notebook);
 
   return session_window;
@@ -685,8 +691,8 @@ save_data_free (SaveData *data)
 }
 
 static int
-write_tab (xmlTextWriterPtr writer,
-           SessionTab      *tab)
+write_tab (xmlTextWriterPtr  writer,
+           SessionTab       *tab)
 {
   int ret;
 
@@ -712,6 +718,14 @@ write_tab (xmlTextWriterPtr writer,
       return ret;
   }
 
+  if (tab->pinned) {
+    ret = xmlTextWriterWriteAttribute (writer,
+                                       (const xmlChar *)"pinned",
+                                       (const xmlChar *)"true");
+    if (ret < 0)
+      return ret;
+  }
+
   if (tab->crashed) {
     ret = xmlTextWriterWriteAttribute (writer,
                                        (const xmlChar *)"crashed",
@@ -744,8 +758,10 @@ write_tab (xmlTextWriterPtr writer,
 }
 
 static int
-write_window_geometry (xmlTextWriterPtr writer,
-                       GdkRectangle    *geometry)
+write_window_geometry (xmlTextWriterPtr  writer,
+                       GdkRectangle     *geometry,
+                       gboolean          is_maximized,
+                       gboolean          is_fullscreen)
 {
   int ret;
 
@@ -767,12 +783,23 @@ write_window_geometry (xmlTextWriterPtr writer,
 
   ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"height", "%d",
                                            geometry->height);
+  if (ret < 0)
+    return ret;
+
+  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"is-maximized", "%d",
+                                           is_maximized);
+  if (ret < 0)
+    return ret;
+
+  ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"is-fullscreen", "%d",
+                                           is_fullscreen);
+
   return ret;
 }
 
 static int
-write_ephy_window (xmlTextWriterPtr writer,
-                   SessionWindow   *window)
+write_ephy_window (xmlTextWriterPtr  writer,
+                   SessionWindow    *window)
 {
   GList *l;
   int ret;
@@ -781,7 +808,7 @@ write_ephy_window (xmlTextWriterPtr writer,
   if (ret < 0)
     return ret;
 
-  ret = write_window_geometry (writer, &window->geometry);
+  ret = write_window_geometry (writer, &window->geometry, window->is_maximized, window->is_fullscreen);
   if (ret < 0)
     return ret;
 
@@ -848,7 +875,9 @@ session_seems_sane (GList *windows)
       if (uri) {
         if (uri->host != NULL ||
             uri->scheme == SOUP_URI_SCHEME_DATA ||
-            uri->scheme == SOUP_URI_SCHEME_FILE)
+            uri->scheme == SOUP_URI_SCHEME_FILE ||
+            strcmp (uri->scheme, "ephy-reader") == 0 ||
+            strcmp (uri->scheme, "ephy-pdf") == 0)
           sane = TRUE;
         soup_uri_free (uri);
       }
@@ -875,14 +904,6 @@ save_session_sync (GTask        *task,
   GList *w;
   int ret = -1;
 
-  /* If any web view has an insane URL, then something has probably gone wrong
-   * inside WebKit. For instance, if the web process is nonfunctional, the UI
-   * process could have an invalid URI property. Yes, this would be a WebKit
-   * bug, but Epiphany should be robust to such issues. Do not clobber an
-   * existing good session file with our new bogus state. Bug #768250. */
-  if (!session_seems_sane (data->windows))
-    return;
-
   buffer = xmlBufferCreate ();
   writer = xmlNewTextWriterMemory (buffer, 0);
   if (writer == NULL)
@@ -920,7 +941,7 @@ save_session_sync (GTask        *task,
 
   ret = xmlTextWriterEndDocument (writer);
 
- out:
+out:
   if (writer)
     xmlFreeTextWriter (writer);
 
@@ -974,6 +995,19 @@ ephy_session_save_idle_cb (EphySession *session)
 
   session->save_source_id = 0;
 
+  /* If we have never successfully loaded any page, or any web view has an
+   * insane URL, then something has probably gone wrong inside WebKit. For
+   * instance, if the web process is nonfunctional, the UI process could have
+   * an invalid URI property. Yes, this would be a WebKit bug, but Epiphany
+   * should be robust to such issues. Do not clobber an existing good session
+   * file with our new bogus state. Bug #768250.
+   */
+  data = save_data_new (session);
+  if (!session->loaded_page || !session_seems_sane (data->windows)) {
+    save_data_free (data);
+    return G_SOURCE_REMOVE;
+  }
+
   if (session->save_cancellable) {
     g_cancellable_cancel (session->save_cancellable);
     g_object_unref (session->save_cancellable);
@@ -984,12 +1018,12 @@ ephy_session_save_idle_cb (EphySession *session)
 
   if (ephy_shell_get_n_windows (shell) == 0) {
     session_delete (session);
+    save_data_free (data);
     return G_SOURCE_REMOVE;
   }
 
   g_application_hold (G_APPLICATION (ephy_shell_get_default ()));
   session->save_cancellable = g_cancellable_new ();
-  data = save_data_new (session);
   task = g_task_new (session, session->save_cancellable,
                      save_session_in_thread_finished_cb, NULL);
   g_task_set_task_data (task, data, (GDestroyNotify)save_data_free);
@@ -1006,18 +1040,15 @@ ephy_session_save (EphySession *session)
 
   g_assert (EPHY_IS_SESSION (session));
 
-  if (session->save_source_id) {
+  if (session->save_source_id)
     return;
-  }
 
-  if (session->dont_save) {
+  if (session->dont_save)
     return;
-  }
 
   policy = g_settings_get_enum (EPHY_SETTINGS_MAIN, EPHY_PREFS_RESTORE_SESSION_POLICY);
-  if (policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER) {
+  if (policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER)
     return;
-  }
 
   session->save_source_id = g_timeout_add_seconds_full (G_PRIORITY_DEFAULT_IDLE, 1,
                                                         (GSourceFunc)ephy_session_save_idle_cb,
@@ -1026,7 +1057,9 @@ ephy_session_save (EphySession *session)
 }
 
 static void
-confirm_before_recover (EphyWindow *window, const char *url, const char *title)
+confirm_before_recover (EphyWindow *window,
+                        const char *url,
+                        const char *title)
 {
   EphyEmbed *embed;
 
@@ -1056,6 +1089,7 @@ typedef struct {
   guint32 user_time;
 
   EphyWindow *window;
+  gulong destroy_id;
   gboolean is_first_window;
   gint active_tab;
 
@@ -1081,18 +1115,31 @@ session_parser_context_free (SessionParserContext *context)
 {
   g_object_unref (context->session);
 
+  if (context->window) {
+    /* This can only happen if the session state is malformed. */
+    g_signal_handler_disconnect (context->window, context->destroy_id);
+  }
+
   g_free (context);
 }
 
 static void
-session_parse_window (SessionParserContext *context,
-                      const gchar         **names,
-                      const gchar         **values)
+session_parse_window (SessionParserContext  *context,
+                      const gchar          **names,
+                      const gchar          **values)
 {
   GdkRectangle geometry = { -1, -1, 0, 0 };
+  gboolean is_maximized = FALSE;
+  gboolean is_fullscreen = FALSE;
   guint i;
 
+  if (context->window) {
+    /* This should only happen if the session state is malformed. */
+    return;
+  }
+
   context->window = ephy_window_new ();
+  context->destroy_id = g_signal_connect (context->window, "destroy", G_CALLBACK (gtk_widget_destroyed), &context->window);
 
   for (i = 0; names[i]; i++) {
     gulong int_value;
@@ -1109,6 +1156,12 @@ session_parse_window (SessionParserContext *context,
     } else if (strcmp (names[i], "height") == 0) {
       ephy_string_to_int (values[i], &int_value);
       geometry.height = int_value;
+    } else if (strcmp (names[i], "is-maximized") == 0) {
+      ephy_string_to_int (values[i], &int_value);
+      is_maximized = int_value != 0;
+    } else if (strcmp (names[i], "is-fullscreen") == 0) {
+      ephy_string_to_int (values[i], &int_value);
+      is_fullscreen = int_value != 0;
     } else if (strcmp (names[i], "role") == 0) {
       gtk_window_set_role (GTK_WINDOW (context->window), values[i]);
     } else if (strcmp (names[i], "active-tab") == 0) {
@@ -1118,21 +1171,38 @@ session_parse_window (SessionParserContext *context,
   }
 
   restore_geometry (GTK_WINDOW (context->window), &geometry);
+
+  if (is_maximized)
+    gtk_window_maximize (GTK_WINDOW (context->window));
+
+  if (is_fullscreen)
+    gtk_window_fullscreen (GTK_WINDOW (context->window));
 }
 
 static void
-session_parse_embed (SessionParserContext *context,
-                     const gchar         **names,
-                     const gchar         **values)
+session_parse_embed (SessionParserContext  *context,
+                     const gchar          **names,
+                     const gchar          **values)
 {
+  GtkWidget *notebook;
   const char *url = NULL;
   const char *title = NULL;
   const char *history = NULL;
   gboolean was_loading = FALSE;
   gboolean crashed = FALSE;
   gboolean is_blank_page = FALSE;
+  gboolean is_pin = FALSE;
   guint i;
 
+  if (!context->window) {
+    /* This can happen if the session is malformed, or if the window is
+     * destroyed before the session finishes loading.
+     */
+    return;
+  }
+
+  notebook = ephy_window_get_notebook (context->window);
+
   for (i = 0; names[i]; i++) {
     if (strcmp (names[i], "url") == 0) {
       url = values[i];
@@ -1146,6 +1216,8 @@ session_parse_embed (SessionParserContext *context,
       crashed = strcmp (values[i], "true") == 0;
     } else if (strcmp (names[i], "history") == 0) {
       history = values[i];
+    } else if (strcmp (names[i], "pinned") == 0) {
+      is_pin = strcmp (values[i], "true") == 0;
     }
   }
 
@@ -1179,6 +1251,8 @@ session_parse_embed (SessionParserContext *context,
                                      context->window, NULL, flags,
                                      0);
 
+    ephy_notebook_tab_set_pinned (EPHY_NOTEBOOK (notebook), GTK_WIDGET (embed), is_pin);
+
     web_view = ephy_embed_get_web_view (embed);
     if (history) {
       guchar *data;
@@ -1227,12 +1301,12 @@ session_parse_embed (SessionParserContext *context,
 }
 
 static void
-session_start_element (GMarkupParseContext *ctx,
-                       const gchar         *element_name,
-                       const gchar        **names,
-                       const gchar        **values,
-                       gpointer             user_data,
-                       GError             **error)
+session_start_element (GMarkupParseContext  *ctx,
+                       const gchar          *element_name,
+                       const gchar         **names,
+                       const gchar         **values,
+                       gpointer              user_data,
+                       GError              **error)
 {
   SessionParserContext *context = (SessionParserContext *)user_data;
 
@@ -1245,10 +1319,10 @@ session_start_element (GMarkupParseContext *ctx,
 }
 
 static void
-session_end_element (GMarkupParseContext *ctx,
-                     const gchar         *element_name,
-                     gpointer             user_data,
-                     GError             **error)
+session_end_element (GMarkupParseContext  *ctx,
+                     const gchar          *element_name,
+                     gpointer              user_data,
+                     GError              **error)
 {
   SessionParserContext *context = (SessionParserContext *)user_data;
 
@@ -1256,6 +1330,13 @@ session_end_element (GMarkupParseContext *ctx,
     GtkWidget *notebook;
     EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
+    if (!context->window) {
+      /* This can happen if the session is malformed, or if the window is
+       * destroyed before the session finishes loading.
+       */
+      return;
+    }
+
     notebook = ephy_window_get_notebook (context->window);
     gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), context->active_tab);
 
@@ -1264,11 +1345,13 @@ session_end_element (GMarkupParseContext *ctx,
 
       active_child = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (context->window));
       gtk_widget_grab_focus (GTK_WIDGET (active_child));
+      ephy_window_update_entry_focus (context->window, ephy_embed_get_web_view (active_child));
       gtk_widget_show (GTK_WIDGET (context->window));
     }
 
     ephy_embed_shell_restored_window (shell);
 
+    g_clear_signal_handler (&context->destroy_id, context->window);
     context->window = NULL;
     context->active_tab = 0;
     context->is_first_window = FALSE;
@@ -1414,12 +1497,12 @@ load_stream_read_cb (GObject      *object,
  * the operation.
  **/
 void
-ephy_session_load_from_stream (EphySession        *session,
-                               GInputStream       *stream,
-                               guint32             user_time,
-                               GCancellable       *cancellable,
-                               GAsyncReadyCallback callback,
-                               gpointer            user_data)
+ephy_session_load_from_stream (EphySession         *session,
+                               GInputStream        *stream,
+                               guint32              user_time,
+                               GCancellable        *cancellable,
+                               GAsyncReadyCallback  callback,
+                               gpointer             user_data)
 {
   GTask *task;
   SessionParserContext *context;
@@ -1461,9 +1544,9 @@ ephy_session_load_from_stream (EphySession        *session,
  * Returns: %TRUE if at least a window has been opened
  **/
 gboolean
-ephy_session_load_from_stream_finish (EphySession  *session,
-                                      GAsyncResult *result,
-                                      GError      **error)
+ephy_session_load_from_stream_finish (EphySession   *session,
+                                      GAsyncResult  *result,
+                                      GError       **error)
 {
   g_assert (g_task_is_valid (result, session));
 
@@ -1554,12 +1637,12 @@ session_read_cb (GObject      *object,
  * the operation.
  **/
 void
-ephy_session_load (EphySession        *session,
-                   const char         *filename,
-                   guint32             user_time,
-                   GCancellable       *cancellable,
-                   GAsyncReadyCallback callback,
-                   gpointer            user_data)
+ephy_session_load (EphySession         *session,
+                   const char          *filename,
+                   guint32              user_time,
+                   GCancellable        *cancellable,
+                   GAsyncReadyCallback  callback,
+                   gpointer             user_data)
 {
   GFile *save_to_file;
   GTask *task;
@@ -1597,9 +1680,9 @@ ephy_session_load (EphySession        *session,
  * Returns: %TRUE if at least a window has been opened
  **/
 gboolean
-ephy_session_load_finish (EphySession  *session,
-                          GAsyncResult *result,
-                          GError      **error)
+ephy_session_load_finish (EphySession   *session,
+                          GAsyncResult  *result,
+                          GError       **error)
 {
   g_assert (g_task_is_valid (result, session));
 
@@ -1641,11 +1724,11 @@ session_resumed_cb (GObject      *object,
 }
 
 void
-ephy_session_resume (EphySession        *session,
-                     guint32             user_time,
-                     GCancellable       *cancellable,
-                     GAsyncReadyCallback callback,
-                     gpointer            user_data)
+ephy_session_resume (EphySession         *session,
+                     guint32              user_time,
+                     GCancellable        *cancellable,
+                     GAsyncReadyCallback  callback,
+                     gpointer             user_data)
 {
   GTask *task;
   gboolean has_session_state;
@@ -1663,14 +1746,15 @@ ephy_session_resume (EphySession        *session,
 
   shell = ephy_shell_get_default ();
 
+  /* If we are auto-resuming, and we never want to
+   * restore the session, clobber the session state
+   * file.
+   */
+  if (policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER)
+    session_delete (session);
+
   if (has_session_state == FALSE ||
       policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER) {
-    /* If we are auto-resuming, and we never want to
-     * restore the session, clobber the session state
-     * file. */
-    if (policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER)
-      session_delete (session);
-
     session_maybe_open_window (session, user_time);
   } else if (ephy_shell_get_n_windows (shell) == 0) {
     ephy_session_load (session, SESSION_STATE, user_time, cancellable,
@@ -1683,9 +1767,9 @@ ephy_session_resume (EphySession        *session,
 }
 
 gboolean
-ephy_session_resume_finish (EphySession  *session,
-                            GAsyncResult *result,
-                            GError      **error)
+ephy_session_resume_finish (EphySession   *session,
+                            GAsyncResult  *result,
+                            GError       **error)
 {
   g_assert (g_task_is_valid (result, session));
 
@@ -1712,3 +1796,9 @@ ephy_session_clear (EphySession *session)
 
   ephy_session_save (session);
 }
+
+gboolean
+ephy_session_is_closing (EphySession *session)
+{
+  return session->closing;
+}
