@@ -24,6 +24,7 @@
 #include "ephy-shell.h"
 
 #include "ephy-debug.h"
+#include "ephy-desktop-utils.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
@@ -34,19 +35,21 @@
 #include "ephy-lockdown.h"
 #include "ephy-notification.h"
 #include "ephy-prefs.h"
+#include "ephy-prefs-dialog.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-sync-utils.h"
 #include "ephy-title-box.h"
 #include "ephy-title-widget.h"
 #include "ephy-type-builtins.h"
+#include "ephy-web-app-utils.h"
 #include "ephy-web-view.h"
 #include "ephy-window.h"
-#include "prefs-dialog.h"
 #include "window-commands.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <handy.h>
 
 struct _EphyShell {
   EphyEmbedShell parent_instance;
@@ -64,7 +67,9 @@ struct _EphyShell {
   EphyShellStartupContext *remote_startup_context;
   GSList *open_uris_idle_ids;
 
+  GHashTable *notifications;
   gchar *open_notification_id;
+  gboolean startup_finished;
 };
 
 static EphyShell *ephy_shell = NULL;
@@ -119,6 +124,9 @@ ephy_shell_startup_continue (EphyShell               *shell,
   EphySession *session = ephy_shell_get_session (shell);
   gboolean new_window_option = (ctx->startup_mode == EPHY_STARTUP_NEW_WINDOW);
   GtkWindow *active_window = gtk_application_get_active_window (GTK_APPLICATION (shell));
+  EphyEmbedShellMode mode;
+
+  mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell));
 
   if (ctx->session_filename != NULL) {
     g_assert (session != NULL);
@@ -138,11 +146,13 @@ ephy_shell_startup_continue (EphyShell               *shell,
 
     ephy_shell_open_uris (shell, uris, ctx->startup_mode, ctx->user_time);
     g_free (homepage_url);
-  } else if (active_window && !ctx->arguments) {
-    /* If the application already has an active window and the --new-window */
-    /* option was not passed, then we should just present it */
+  } else if (active_window && (!ctx->arguments || mode == EPHY_EMBED_SHELL_MODE_APPLICATION)) {
+    /* If the application already has an active window and: */
+    /*   Option 1: the --new-window option was not passed */
+    /*   Option 2: mode is application */
+    /* then we should just present it */
     /* This can happen for example if the user starts a long download and then */
-    /* closes the browser.*/
+    /* closes the browser or the web app is running in background .*/
     /* The window will still remain active and presenting it will have a */
     /* session-resumed feel for the user. */
     gtk_window_present (active_window);
@@ -155,6 +165,8 @@ ephy_shell_startup_continue (EphyShell               *shell,
     EphyWindow *window = ephy_window_new ();
     ephy_link_open (EPHY_LINK (window), NULL, NULL, EPHY_LINK_HOME_PAGE);
   }
+
+  shell->startup_finished = TRUE;
 }
 
 static void
@@ -205,6 +217,18 @@ export_bookmarks (GSimpleAction *action,
   window_cmd_export_bookmarks (NULL, NULL, EPHY_WINDOW (window));
 }
 
+static void
+import_passwords (GSimpleAction *action,
+                  GVariant      *parameter,
+                  gpointer       user_data)
+{
+  GtkWindow *window;
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (ephy_shell));
+
+  window_cmd_import_passwords (NULL, NULL, EPHY_WINDOW (window));
+}
+
 static void
 show_history (GSimpleAction *action,
               GVariant      *parameter,
@@ -289,11 +313,41 @@ show_downloads (GSimpleAction *action,
   g_signal_emit_by_name (manager, "show-downloads", NULL);
 }
 
+static void
+launch_app (GSimpleAction *action,
+            GVariant      *parameter,
+            gpointer       user_data)
+{
+  const gchar *desktop_file = g_variant_get_string (parameter, NULL);
+
+  /* We can't get here under flatpak because all web app functionality
+   * is disabled when running under flatpak.
+   */
+  ephy_file_launch_desktop_file (desktop_file,
+                                 gtk_get_current_event_time (),
+                                 EPHY_FILE_HELPERS_I_UNDERSTAND_I_MUST_NOT_USE_THIS_FUNCTION_UNDER_FLATPAK);
+}
+
+static void
+notification_clicked (GSimpleAction *action,
+                      GVariant      *parameter,
+                      gpointer       user_data)
+{
+  EphyShell *shell = ephy_shell_get_default ();
+  guint64 id = g_variant_get_uint64 (parameter);
+  WebKitNotification *notification;
+
+  notification = g_hash_table_lookup (shell->notifications, GINT_TO_POINTER (id));
+  if (notification)
+    webkit_notification_clicked (notification);
+}
+
 static GActionEntry app_entries[] = {
   { "new-window", new_window, NULL, NULL, NULL },
   { "new-incognito", new_incognito_window, NULL, NULL, NULL },
   { "import-bookmarks", import_bookmarks, NULL, NULL, NULL },
   { "export-bookmarks", export_bookmarks, NULL, NULL, NULL },
+  { "import-passwords", import_passwords, NULL, NULL, NULL },
   { "history", show_history, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "shortcuts", show_shortcuts, NULL, NULL, NULL },
@@ -301,6 +355,8 @@ static GActionEntry app_entries[] = {
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
   { "show-downloads", show_downloads, NULL, NULL, NULL },
+  { "launch-app", launch_app, "s", NULL, NULL },
+  { "notification-clicked", notification_clicked, "t", NULL, NULL},
 };
 
 static GActionEntry non_incognito_extra_app_entries[] = {
@@ -312,6 +368,8 @@ static GActionEntry app_mode_app_entries[] = {
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
+  { "run-in-background", NULL, NULL, "false", NULL},
+  { "notification-clicked", notification_clicked, "t", NULL, NULL},
 };
 
 static void
@@ -390,15 +448,37 @@ set_accel_for_action (EphyShell   *shell,
   gtk_application_set_accels_for_action (GTK_APPLICATION (shell), detailed_action_name, accels);
 }
 
+static gboolean
+run_in_background_get_mapping (GValue   *value,
+                               GVariant *variant,
+                               gpointer  user_data)
+{
+  g_value_set_variant (value, variant);
+  return TRUE;
+}
+
+static GVariant *
+run_in_background_set_mapping (const GValue       *value,
+                               const GVariantType *expected_type,
+                               gpointer            user_data)
+{
+  GVariant *var = g_value_get_variant (value);
+
+  return g_variant_new_boolean (g_variant_get_boolean (var));
+}
+
 static void
 ephy_shell_startup (GApplication *application)
 {
   EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (application);
   EphyShell *shell = EPHY_SHELL (application);
   EphyEmbedShellMode mode;
+  GAction *action;
 
   G_APPLICATION_CLASS (ephy_shell_parent_class)->startup (application);
 
+  hdy_init ();
+
   /* We're not remoting; start our services */
 
   mode = ephy_embed_shell_get_mode (embed_shell);
@@ -431,12 +511,23 @@ ephy_shell_startup (GApplication *application)
     set_accel_for_action (shell, "app.reopen-closed-tab", "<Primary><Shift>t");
     set_accel_for_action (shell, "app.import-bookmarks", "<Primary><Shift>m");
     set_accel_for_action (shell, "app.export-bookmarks", "<Primary><Shift>x");
-    set_accel_for_action (shell, "app.shortcuts", "<Primary>F1");
+    set_accel_for_action (shell, "app.shortcuts", "<Primary>question");
     set_accel_for_action (shell, "app.help", "F1");
   } else {
     g_action_map_add_action_entries (G_ACTION_MAP (application),
                                      app_mode_app_entries, G_N_ELEMENTS (app_mode_app_entries),
                                      application);
+
+    action = g_action_map_lookup_action (G_ACTION_MAP (application), "run-in-background");
+    g_settings_bind_with_mapping (EPHY_SETTINGS_WEB_APP,
+                                  EPHY_PREFS_WEB_APP_RUN_IN_BACKGROUND,
+                                  action,
+                                  "state",
+                                  G_SETTINGS_BIND_DEFAULT,
+                                  run_in_background_get_mapping,
+                                  run_in_background_set_mapping,
+                                  NULL,
+                                  NULL);
   }
 
   /* Actions that are available in both app mode and browser mode */
@@ -495,12 +586,35 @@ session_load_cb (GObject      *object,
   ephy_shell_startup_continue (ephy_shell_get_default (), ctx);
 }
 
+static void
+portal_check (EphyShell *shell)
+{
+  if (g_network_monitor_get_connectivity (ephy_shell_get_net_monitor (shell)) == G_NETWORK_CONNECTIVITY_PORTAL) {
+    EphyWindow *window = EPHY_WINDOW (gtk_application_get_active_window (GTK_APPLICATION (shell)));
+
+    ephy_link_open (EPHY_LINK (window), "http://nmcheck.gnome.org/", NULL, EPHY_LINK_NEW_TAB | EPHY_LINK_JUMP_TO);
+  }
+}
+
+static void
+connectivity_changed (GNetworkMonitor *monitor,
+                      GParamSpec      *pspec,
+                      EphyShell       *shell)
+{
+  portal_check (shell);
+}
+
 static void
 ephy_shell_activate (GApplication *application)
 {
   EphyShell *shell = EPHY_SHELL (application);
   EphyEmbedShell *embed_shell = EPHY_EMBED_SHELL (shell);
 
+  if (!is_desktop_gnome ()) {
+    g_signal_connect (ephy_shell_get_net_monitor (shell), "notify::connectivity", G_CALLBACK (connectivity_changed), shell);
+    portal_check (shell);
+  }
+
   if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_AUTOMATION) {
     WebKitWebContext *web_context = ephy_embed_shell_get_web_context (embed_shell);
     g_signal_connect (web_context, "automation-started", G_CALLBACK (automation_started_cb), shell);
@@ -667,7 +781,8 @@ ephy_shell_get_lockdown (EphyShell *shell)
 static void
 ephy_shell_constructed (GObject *object)
 {
-  if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (object)) != EPHY_EMBED_SHELL_MODE_BROWSER) {
+  if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (object)) != EPHY_EMBED_SHELL_MODE_BROWSER &&
+      ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (object)) != EPHY_EMBED_SHELL_MODE_APPLICATION) {
     GApplicationFlags flags;
 
     flags = g_application_get_flags (G_APPLICATION (object));
@@ -711,8 +826,11 @@ ephy_shell_init (EphyShell *shell)
   /* globally accessible singleton */
   g_assert (ephy_shell == NULL);
   ephy_shell = shell;
+  ephy_shell->startup_finished = FALSE;
   g_object_add_weak_pointer (G_OBJECT (ephy_shell),
                              (gpointer *)ptr);
+
+  ephy_shell->notifications = g_hash_table_new (g_direct_hash, g_direct_equal);
 }
 
 static void
@@ -738,6 +856,8 @@ ephy_shell_dispose (GObject *object)
   g_clear_object (&shell->history_manager);
   g_clear_object (&shell->open_tabs_manager);
 
+  g_hash_table_destroy (shell->notifications);
+
   if (shell->open_notification_id) {
     g_application_withdraw_notification (G_APPLICATION (shell), shell->open_notification_id);
     g_clear_pointer (&shell->open_notification_id, g_free);
@@ -775,6 +895,85 @@ ephy_shell_get_default (void)
   return ephy_shell;
 }
 
+static void
+webkit_notification_clicked_cb (WebKitNotification *notification,
+                                gpointer            user_data)
+{
+  WebKitWebView *notification_webview = WEBKIT_WEB_VIEW (user_data);
+  EphyShell *shell;
+  GApplication *application;
+  GList *windows;
+
+  shell = ephy_shell_get_default ();
+  application = G_APPLICATION (shell);
+  windows = gtk_application_get_windows (GTK_APPLICATION (application));
+
+  for (guint win_idx = 0; win_idx < g_list_length (windows); win_idx++) {
+    EphyWindow *window;
+    GtkWidget *notebook;
+    int n_pages;
+
+    window = EPHY_WINDOW (g_list_nth_data (windows, win_idx));
+
+    notebook = ephy_window_get_notebook (window);
+    n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+
+    for (int i = 0; i < n_pages; i++) {
+      EphyEmbed *embed;
+      WebKitWebView *webview;
+
+      embed = EPHY_EMBED (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), i));
+      webview = WEBKIT_WEB_VIEW (ephy_embed_get_web_view (embed));
+
+      if (webview == notification_webview) {
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), i);
+        gtk_window_present (GTK_WINDOW (window));
+        return;
+      }
+    }
+  }
+}
+
+static void
+webkit_notification_closed_cb (WebKitNotification *notification,
+                               gpointer            user_data)
+{
+  EphyShell *shell = ephy_shell_get_default ();
+  g_autofree char *id = NULL;
+
+  id = g_strdup_printf ("%" G_GUINT64_FORMAT, webkit_notification_get_id (notification));
+
+  g_application_withdraw_notification (G_APPLICATION (g_application_get_default ()), id);
+  g_hash_table_remove (shell->notifications, GINT_TO_POINTER (webkit_notification_get_id (notification)));
+}
+
+static gboolean
+show_notification_cb (WebKitWebView      *web_view,
+                      WebKitNotification *notification,
+                      gpointer            user_data)
+{
+  EphyShell *shell = ephy_shell_get_default ();
+  GNotification *notify;
+  g_autofree char *id = NULL;
+
+  id = g_strdup_printf ("%" G_GUINT64_FORMAT, webkit_notification_get_id (notification));
+
+  g_signal_connect_object (notification, "clicked", G_CALLBACK (webkit_notification_clicked_cb), web_view, 0);
+  g_signal_connect_object (notification, "closed", G_CALLBACK (webkit_notification_closed_cb), web_view, 0);
+
+  notify = g_notification_new (webkit_notification_get_title (notification));
+  g_notification_set_body (notify, webkit_notification_get_body (notification));
+
+  g_notification_set_priority (notify, G_NOTIFICATION_PRIORITY_LOW);
+  g_notification_set_default_action_and_target (notify, "app.notification-clicked", "t", webkit_notification_get_id (notification));
+
+  g_hash_table_insert (shell->notifications, GINT_TO_POINTER (webkit_notification_get_id (notification)), notification);
+
+  g_application_send_notification (G_APPLICATION (g_application_get_default ()), id, notify);
+
+  return TRUE;
+}
+
 /**
  * ephy_shell_new_tab_full:
  * @shell: a #EphyShell
@@ -829,6 +1028,8 @@ ephy_shell_new_tab_full (EphyShell       *shell,
   else
     web_view = ephy_web_view_new ();
 
+  g_signal_connect (web_view, "show-notification", G_CALLBACK (show_notification_cb), NULL);
+
   embed = EPHY_EMBED (g_object_new (EPHY_TYPE_EMBED,
                                     "web-view", web_view,
                                     "title", title,
@@ -1024,9 +1225,8 @@ GObject *
 ephy_shell_get_prefs_dialog (EphyShell *shell)
 {
   if (shell->prefs_dialog == NULL) {
-    shell->prefs_dialog = g_object_new (EPHY_TYPE_PREFS_DIALOG,
-                                        "use-header-bar", TRUE,
-                                        NULL);
+    shell->prefs_dialog = g_object_new (EPHY_TYPE_PREFS_DIALOG, NULL);
+
     g_signal_connect (shell->prefs_dialog,
                       "destroy",
                       G_CALLBACK (gtk_widget_destroyed),
@@ -1039,14 +1239,18 @@ ephy_shell_get_prefs_dialog (EphyShell *shell)
 void
 _ephy_shell_create_instance (EphyEmbedShellMode mode)
 {
-  g_autofree gchar *id = NULL;
+  const char *id = NULL;
 
   g_assert (ephy_shell == NULL);
 
-  if (mode == EPHY_EMBED_SHELL_MODE_APPLICATION)
-    id = g_strconcat (APPLICATION_ID, ".WebApp", NULL);
-  else
-    id = g_strdup (APPLICATION_ID);
+  if (mode == EPHY_EMBED_SHELL_MODE_APPLICATION) {
+    const char *profile_dir = ephy_profile_dir ();
+    const char *web_id = ephy_web_application_get_program_name_from_profile_directory (profile_dir);
+
+    id = web_id;
+  } else {
+    id = APPLICATION_ID;
+  }
 
   ephy_shell = EPHY_SHELL (g_object_new (EPHY_TYPE_SHELL,
                                          "application-id", id,
@@ -1282,3 +1486,9 @@ ephy_shell_send_notification (EphyShell     *shell,
   shell->open_notification_id = g_strdup (id);
   g_application_send_notification (G_APPLICATION (shell), id, notification);
 }
+
+gboolean
+ephy_shell_startup_finished (EphyShell *shell)
+{
+  return shell->startup_finished;
+}
