@@ -7,7 +7,7 @@ fi
 
 SASSC_OPT="-M -t compact"
 
-: ${GTK_SOURCE_PATH:="../../../gtk"}
+: ${GTK_SOURCE_PATH:="../../../gtk+-3"}
 
 sassc $SASSC_OPT -I${GTK_SOURCE_PATH}/gtk/theme/Adwaita \
 	themes/Adwaita.scss themes/Adwaita.css
@@ -17,3 +17,9 @@ sassc $SASSC_OPT -I${GTK_SOURCE_PATH}/gtk/theme/Adwaita \
 	themes/elementary.scss themes/elementary.css
 sassc $SASSC_OPT -I${GTK_SOURCE_PATH}/gtk/theme/Adwaita \
 	themes/shared.scss themes/shared.css
+sassc $SASSC_OPT -I${GTK_SOURCE_PATH}/gtk/theme/Adwaita \
+	-I${GTK_SOURCE_PATH}/gtk/theme/HighContrast \
+	themes/HighContrast.scss themes/HighContrast.css
+sassc $SASSC_OPT -I${GTK_SOURCE_PATH}/gtk/theme/Adwaita \
+	-I${GTK_SOURCE_PATH}/gtk/theme/HighContrast \
+	themes/HighContrastInverse.scss themes/HighContrastInverse.css
