@@ -32,8 +32,7 @@ typedef struct {
   EphyFileHelpersFlags flags;
 } FileInitTest;
 
-static const FileInitTest private_tests[] =
-{
+static const FileInitTest private_tests[] = {
   { "private", EPHY_FILE_HELPERS_PRIVATE_PROFILE },
   { "private, keep-dir", EPHY_FILE_HELPERS_PRIVATE_PROFILE | EPHY_FILE_HELPERS_KEEP_DIR }
 };
@@ -47,7 +46,7 @@ test_ephy_file_helpers_init (void)
     FileInitTest test;
 
     char *tmp_dir = NULL;
-    char *dot_dir = NULL;
+    char *profile_dir = NULL;
 
     gboolean private_profile = FALSE;
     gboolean keep_dir = FALSE;
@@ -69,20 +68,20 @@ test_ephy_file_helpers_init (void)
     g_assert_true (ephy_file_helpers_init (NULL, test.flags, NULL));
 
     tmp_dir = g_strdup (ephy_file_tmp_dir ());
-    dot_dir = g_strdup (ephy_profile_dir ());
+    profile_dir = g_strdup (ephy_profile_dir ());
 
     g_assert_nonnull (tmp_dir);
-    g_assert_nonnull (dot_dir);
+    g_assert_nonnull (profile_dir);
 
     /* Should always exist after ephy_file_tmp_dir(). */
     g_assert_true (g_file_test (tmp_dir, G_FILE_TEST_EXISTS));
-    g_assert_true (g_file_test (dot_dir, G_FILE_TEST_EXISTS) == ensure_exists);
+    g_assert_true (g_file_test (profile_dir, G_FILE_TEST_EXISTS) == ensure_exists);
 
     ephy_file_helpers_shutdown ();
 
-    /* Private profiles have their dot_dir inside tmp_dir. */
+    /* Private profiles have their profile_dir inside tmp_dir. */
     g_assert_true (g_file_test (tmp_dir, G_FILE_TEST_EXISTS) == keep_dir);
-    g_assert_true (g_file_test (dot_dir, G_FILE_TEST_EXISTS) == (keep_dir && ensure_exists));
+    g_assert_true (g_file_test (profile_dir, G_FILE_TEST_EXISTS) == (keep_dir && ensure_exists));
 
     /* Cleanup dir left behind. */
     if (keep_dir) {
@@ -95,7 +94,7 @@ test_ephy_file_helpers_init (void)
     }
 
     g_free (tmp_dir);
-    g_free (dot_dir);
+    g_free (profile_dir);
   }
 }
 
@@ -106,8 +105,7 @@ typedef struct {
   gboolean in_flatpak;
 } DownloadsDirTest;
 
-static const DownloadsDirTest downloads_tests[] =
-{
+static const DownloadsDirTest downloads_tests[] = {
   { "Desktop", NULL, G_USER_DIRECTORY_DESKTOP, FALSE },
 
   { "Downloads", NULL, G_USER_DIRECTORY_DOWNLOAD, TRUE },
@@ -163,8 +161,7 @@ typedef struct {
   gboolean can_delete;
 } DirTest;
 
-static const DirTest dir_tests[] =
-{
+static const DirTest dir_tests[] = {
   { "/tmp/ephy-test-dir", FALSE, TRUE, TRUE },
   /* Unreadable */
   { "/root/ephy-test-dir", FALSE, FALSE, FALSE },
@@ -222,7 +219,7 @@ test_ephy_file_desktop_dir (void)
 static void
 delete_uri (const char *uri)
 {
-  g_autoptr(GFile) file = NULL;
+  g_autoptr (GFile) file = NULL;
 
   g_assert_nonnull (uri);
 
@@ -300,8 +297,7 @@ typedef struct {
   const char *expected;
 } SanitizeFilenameTest;
 
-static const SanitizeFilenameTest sanitize_filename_tests[] =
-{
+static const SanitizeFilenameTest sanitize_filename_tests[] = {
   { "Normal Filename", "Normal Filename" },
   { "filename/with/slashes", "filename_with_slashes" }
 };
@@ -329,7 +325,8 @@ test_ephy_sanitize_filename (void)
 }
 
 int
-main (int argc, char *argv[])
+main (int   argc,
+      char *argv[])
 {
   int ret;
 
